import logging
from .auth import AuthPhase as AuthPhase, auth_required_message as auth_required_message
from .const import CANCELLATION_ERRORS as CANCELLATION_ERRORS, DATA_CONNECTIONS as DATA_CONNECTIONS, MAX_PENDING_MSG as MAX_PENDING_MSG, PENDING_MSG_PEAK as PENDING_MSG_PEAK, PENDING_MSG_PEAK_TIME as PENDING_MSG_PEAK_TIME, SIGNAL_WEBSOCKET_CONNECTED as SIGNAL_WEBSOCKET_CONNECTED, SIGNAL_WEBSOCKET_DISCONNECTED as SIGNAL_WEBSOCKET_DISCONNECTED, URL as URL
from .error import Disconnect as Disconnect
from .messages import message_to_json as message_to_json
from aiohttp import web
from homeassistant.components.http import HomeAssistantView as HomeAssistantView
from homeassistant.const import EVENT_HOMEASSISTANT_STOP as EVENT_HOMEASSISTANT_STOP
from homeassistant.core import callback as callback
from homeassistant.helpers.event import async_call_later as async_call_later
from typing import Any

_WS_LOGGER: Any

class WebsocketAPIView(HomeAssistantView):
    name: str = ...
    url: Any = ...
    requires_auth: bool = ...
    async def get(self, request: web.Request) -> web.WebSocketResponse: ...

class WebSocketAdapter(logging.LoggerAdapter):
    def process(self, msg: Any, kwargs: Any): ...

class WebSocketHandler:
    hass: Any = ...
    request: Any = ...
    wsock: Any = ...
    _to_write: Any = ...
    _handle_task: Any = ...
    _writer_task: Any = ...
    _logger: Any = ...
    _peak_checker_unsub: Any = ...
    def __init__(self, hass: Any, request: Any) -> None: ...
    async def _writer(self) -> None: ...
    def _send_message(self, message: Any) -> None: ...
    def _check_write_peak(self, _: Any) -> None: ...
    def _cancel(self) -> None: ...
    async def async_handle(self) -> web.WebSocketResponse: ...



import logging
from .debounce import Debouncer as Debouncer
from datetime import datetime, timedelta
from homeassistant.const import EVENT_HOMEASSISTANT_STOP as EVENT_HOMEASSISTANT_STOP
from homeassistant.core import CALLBACK_TYPE as CALLBACK_TYPE, Event as Event, HassJob as HassJob, HomeAssistant as HomeAssistant, callback as callback
from homeassistant.exceptions import ConfigEntryNotReady as ConfigEntryNotReady
from homeassistant.helpers import entity as entity, event as event
from homeassistant.util.dt import utcnow as utcnow
from typing import Any, Awaitable, Callable, TypeVar

REQUEST_REFRESH_DEFAULT_COOLDOWN: int
REQUEST_REFRESH_DEFAULT_IMMEDIATE: bool
T = TypeVar('T')

class UpdateFailed(Exception): ...

class DataUpdateCoordinator:
    hass: Any = ...
    logger: Any = ...
    name: Any = ...
    update_method: Any = ...
    update_interval: Any = ...
    data: Any = ...
    _listeners: Any = ...
    _job: Any = ...
    _unsub_refresh: Any = ...
    _request_refresh_task: Any = ...
    last_update_success: bool = ...
    last_exception: Any = ...
    _debounced_refresh: Any = ...
    def __init__(self, hass: HomeAssistant, logger: logging.Logger, name: str, *, update_interval: Union[timedelta, None]=..., update_method: Union[Callable[[], Awaitable[T]], None]=..., request_refresh_debouncer: Union[Debouncer, None]=...) -> None: ...
    def async_add_listener(self, update_callback: CALLBACK_TYPE) -> Callable[[], None]: ...
    def async_remove_listener(self, update_callback: CALLBACK_TYPE) -> None: ...
    def _schedule_refresh(self) -> None: ...
    async def _handle_refresh_interval(self, _now: datetime) -> None: ...
    async def async_request_refresh(self) -> None: ...
    async def _async_update_data(self) -> Union[T, None]: ...
    async def async_config_entry_first_refresh(self) -> None: ...
    async def async_refresh(self) -> None: ...
    async def _async_refresh(self, log_failures: bool=...) -> None: ...
    def async_set_updated_data(self, data: T) -> None: ...
    def _async_stop_refresh(self, _: Event) -> None: ...

class CoordinatorEntity(entity.Entity):
    coordinator: Any = ...
    def __init__(self, coordinator: DataUpdateCoordinator[Any]) -> None: ...
    @property
    def should_poll(self) -> bool: ...
    @property
    def available(self) -> bool: ...
    async def async_added_to_hass(self) -> None: ...
    def _handle_coordinator_update(self) -> None: ...
    async def async_update(self) -> None: ...



from . import const as const
from .helpers import async_get_node_from_device_id as async_get_node_from_device_id, async_get_node_from_entity_id as async_get_node_from_entity_id
from homeassistant.const import ATTR_DEVICE_ID as ATTR_DEVICE_ID, ATTR_ENTITY_ID as ATTR_ENTITY_ID
from homeassistant.core import HomeAssistant as HomeAssistant, ServiceCall as ServiceCall, callback as callback
from homeassistant.helpers.dispatcher import async_dispatcher_send as async_dispatcher_send
from homeassistant.helpers.entity_registry import EntityRegistry as EntityRegistry
from typing import Any

_LOGGER: Any

def parameter_name_does_not_need_bitmask(val: dict[str, Union[int, str]]) -> dict[str, Union[int, str]]: ...

BITMASK_SCHEMA: Any

class ZWaveServices:
    _hass: Any = ...
    _ent_reg: Any = ...
    def __init__(self, hass: HomeAssistant, ent_reg: EntityRegistry) -> None: ...
    def async_register(self) -> None: ...
    async def async_set_config_parameter(self, service: ServiceCall) -> None: ...
    async def async_bulk_set_partial_config_parameters(self, service: ServiceCall) -> None: ...
    async def async_poll_value(self, service: ServiceCall) -> None: ...
    async def async_set_value(self, service: ServiceCall) -> None: ...



from homeassistant.const import ATTR_LATITUDE as ATTR_LATITUDE, ATTR_LONGITUDE as ATTR_LONGITUDE, CONF_DEVICES as CONF_DEVICES, CONF_UNIT_OF_MEASUREMENT as CONF_UNIT_OF_MEASUREMENT, CONF_ZONE as CONF_ZONE, LENGTH_FEET as LENGTH_FEET, LENGTH_KILOMETERS as LENGTH_KILOMETERS, LENGTH_METERS as LENGTH_METERS, LENGTH_MILES as LENGTH_MILES, LENGTH_YARD as LENGTH_YARD
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.event import track_state_change as track_state_change
from homeassistant.util.distance import convert as convert
from homeassistant.util.location import distance as distance
from typing import Any

_LOGGER: Any
ATTR_DIR_OF_TRAVEL: str
ATTR_DIST_FROM: str
ATTR_NEAREST: str
CONF_IGNORED_ZONES: str
CONF_TOLERANCE: str
DEFAULT_DIR_OF_TRAVEL: str
DEFAULT_DIST_TO_ZONE: str
DEFAULT_NEAREST: str
DEFAULT_PROXIMITY_ZONE: str
DEFAULT_TOLERANCE: int
DOMAIN: str
UNITS: Any
ZONE_SCHEMA: Any
CONFIG_SCHEMA: Any

def setup_proximity_component(hass: Any, name: Any, config: Any): ...
def setup(hass: Any, config: Any): ...

class Proximity(Entity):
    hass: Any = ...
    friendly_name: Any = ...
    dist_to: Any = ...
    dir_of_travel: Any = ...
    nearest: Any = ...
    ignored_zones: Any = ...
    proximity_devices: Any = ...
    tolerance: Any = ...
    proximity_zone: Any = ...
    _unit_of_measurement: Any = ...
    def __init__(self, hass: Any, zone_friendly_name: Any, dist_to: Any, dir_of_travel: Any, nearest: Any, ignored_zones: Any, proximity_devices: Any, tolerance: Any, proximity_zone: Any, unit_of_measurement: Any) -> None: ...
    @property
    def name(self): ...
    @property
    def state(self): ...
    @property
    def unit_of_measurement(self): ...
    @property
    def extra_state_attributes(self): ...
    def check_proximity_state_change(self, entity: Any, old_state: Any, new_state: Any) -> None: ...



from aiohttp.web_urldispatcher import StaticResource
from typing import Any

CACHE_TIME: Any
CACHE_HEADERS: Any

class CachingStaticResource(StaticResource):
    async def _handle(self, request: Any): ...



from homeassistant.const import CONF_ELEVATION as CONF_ELEVATION, EVENT_CORE_CONFIG_UPDATE as EVENT_CORE_CONFIG_UPDATE, SUN_EVENT_SUNRISE as SUN_EVENT_SUNRISE, SUN_EVENT_SUNSET as SUN_EVENT_SUNSET
from homeassistant.core import callback as callback
from homeassistant.helpers import event as event
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.sun import get_astral_location as get_astral_location, get_location_astral_event_next as get_location_astral_event_next
from typing import Any, Optional

_LOGGER: Any
DOMAIN: str
ENTITY_ID: str
STATE_ABOVE_HORIZON: str
STATE_BELOW_HORIZON: str
STATE_ATTR_AZIMUTH: str
STATE_ATTR_ELEVATION: str
STATE_ATTR_RISING: str
STATE_ATTR_NEXT_DAWN: str
STATE_ATTR_NEXT_DUSK: str
STATE_ATTR_NEXT_MIDNIGHT: str
STATE_ATTR_NEXT_NOON: str
STATE_ATTR_NEXT_RISING: str
STATE_ATTR_NEXT_SETTING: str
PHASE_NIGHT: str
PHASE_ASTRONOMICAL_TWILIGHT: str
PHASE_NAUTICAL_TWILIGHT: str
PHASE_TWILIGHT: str
PHASE_SMALL_DAY: str
PHASE_DAY: str
_PHASE_UPDATES: Any

async def async_setup(hass: Any, config: Any): ...

class Sun(Entity):
    entity_id: Any = ...
    hass: Any = ...
    location: Any = ...
    _state: Any = ...
    next_dawn: Any = ...
    next_midnight: Any = ...
    solar_elevation: Any = ...
    rising: Any = ...
    _next_change: Any = ...
    def __init__(self, hass: Any) -> None: ...
    @property
    def name(self): ...
    @property
    def state(self): ...
    @property
    def extra_state_attributes(self): ...
    phase: Any = ...
    def _check_event(self, utc_point_in_time: Any, sun_event: Any, before: Any): ...
    next_rising: Any = ...
    next_noon: Any = ...
    next_setting: Any = ...
    next_dusk: Any = ...
    def update_events(self, now: Optional[Any] = ...) -> None: ...
    solar_azimuth: Any = ...
    def update_sun_position(self, now: Optional[Any] = ...) -> None: ...



from homeassistant.core import callback as callback
from typing import Any, Generator
from zwave_js_server.model.device_class import DeviceClassItem as DeviceClassItem
from zwave_js_server.model.node import Node as ZwaveNode
from zwave_js_server.model.value import Value as ZwaveValue

class ZwaveDiscoveryInfo:
    node: ZwaveNode
    primary_value: ZwaveValue
    platform: str
    platform_hint: Union[str, None] = ...
    def __init__(self, node: Any, primary_value: Any, platform: Any, platform_hint: Any) -> None: ...

class ZWaveValueDiscoverySchema:
    command_class: Union[set[int], None] = ...
    endpoint: Union[set[int], None] = ...
    property: Union[set[Union[str, int]], None] = ...
    property_name: Union[set[str], None] = ...
    property_key: Union[set[Union[str, int]], None] = ...
    property_key_name: Union[set[str], None] = ...
    type: Union[set[str], None] = ...
    def __init__(self, command_class: Any, endpoint: Any, property: Any, property_name: Any, property_key: Any, property_key_name: Any, type: Any) -> None: ...

class ZWaveDiscoverySchema:
    platform: str
    primary_value: ZWaveValueDiscoverySchema
    hint: Union[str, None] = ...
    manufacturer_id: Union[set[int], None] = ...
    product_id: Union[set[int], None] = ...
    product_type: Union[set[int], None] = ...
    firmware_version: Union[set[str], None] = ...
    device_class_basic: Union[set[Union[str, int]], None] = ...
    device_class_generic: Union[set[Union[str, int]], None] = ...
    device_class_specific: Union[set[Union[str, int]], None] = ...
    required_values: Union[list[ZWaveValueDiscoverySchema], None] = ...
    absent_values: Union[list[ZWaveValueDiscoverySchema], None] = ...
    allow_multi: bool = ...
    def __init__(self, platform: Any, primary_value: Any, hint: Any, manufacturer_id: Any, product_id: Any, product_type: Any, firmware_version: Any, device_class_basic: Any, device_class_generic: Any, device_class_specific: Any, required_values: Any, absent_values: Any, allow_multi: Any) -> None: ...

def get_config_parameter_discovery_schema(property_: Union[set[Union[str, int]], None]=..., property_name: Union[set[str], None]=..., property_key: Union[set[Union[str, int]], None]=..., property_key_name: Union[set[str], None]=..., **kwargs: Any) -> ZWaveDiscoverySchema: ...

SWITCH_MULTILEVEL_CURRENT_VALUE_SCHEMA: Any
DISCOVERY_SCHEMAS: Any

def async_discover_values(node: ZwaveNode) -> Generator[ZwaveDiscoveryInfo, None, None]: ...
def check_value(value: ZwaveValue, schema: ZWaveValueDiscoverySchema) -> bool: ...
def check_device_class(device_class: DeviceClassItem, required_value: Union[set[Union[str, int]], None]) -> bool: ...



from .const import DOMAIN as DOMAIN
from .knx_entity import KnxEntity as KnxEntity
from homeassistant.components.weather import WeatherEntity as WeatherEntity
from homeassistant.const import TEMP_CELSIUS as TEMP_CELSIUS
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.typing import ConfigType as ConfigType, DiscoveryInfoType as DiscoveryInfoType
from typing import Any, Callable, Iterable
from xknx.devices import Weather as XknxWeather

async def async_setup_platform(hass: HomeAssistant, config: ConfigType, async_add_entities: Callable[[Iterable[Entity]], None], discovery_info: Union[DiscoveryInfoType, None]=...) -> None: ...

class KNXWeather(KnxEntity, WeatherEntity):
    _device: Any
    def __init__(self, device: XknxWeather) -> None: ...
    @property
    def temperature(self) -> Union[float, None]: ...
    @property
    def temperature_unit(self) -> str: ...
    @property
    def pressure(self) -> Union[float, None]: ...
    @property
    def condition(self) -> str: ...
    @property
    def humidity(self) -> Union[float, None]: ...
    @property
    def wind_bearing(self) -> Union[int, None]: ...
    @property
    def wind_speed(self) -> Union[float, None]: ...



from . import HuaweiLteBaseEntity as HuaweiLteBaseEntity
from .const import DOMAIN as DOMAIN, KEY_WLAN_HOST_LIST as KEY_WLAN_HOST_LIST, UPDATE_SIGNAL as UPDATE_SIGNAL
from homeassistant.components.device_tracker.config_entry import ScannerEntity as ScannerEntity
from homeassistant.components.device_tracker.const import SOURCE_TYPE_ROUTER as SOURCE_TYPE_ROUTER
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_URL as CONF_URL
from homeassistant.core import callback as callback
from homeassistant.helpers import entity_registry as entity_registry
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.typing import HomeAssistantType as HomeAssistantType
from typing import Any, Callable

_LOGGER: Any
_DEVICE_SCAN: Any

async def async_setup_entry(hass: HomeAssistantType, config_entry: ConfigEntry, async_add_entities: Callable[[list[Entity], bool], None]) -> None: ...
def async_add_new_entities(hass: HomeAssistantType, router_url: str, async_add_entities: Callable[[list[Entity], bool], None], tracked: set[str]) -> None: ...
def _better_snakecase(text: str) -> str: ...

class HuaweiLteScannerEntity(HuaweiLteBaseEntity, ScannerEntity):
    mac: str = ...
    _is_connected: bool = ...
    _hostname: Union[str, None] = ...
    _extra_state_attributes: dict[str, Any] = ...
    def __attrs_post_init__(self) -> None: ...
    @property
    def _entity_name(self) -> str: ...
    @property
    def _device_unique_id(self) -> str: ...
    @property
    def source_type(self) -> str: ...
    @property
    def is_connected(self) -> bool: ...
    @property
    def extra_state_attributes(self) -> dict[str, Any]: ...
    async def async_update(self) -> None: ...
    def __init__(self, router: Any, available: Any, unsub_handlers: Any, mac: Any, is_connected: Any, hostname: Any, extra_state_attributes: Any) -> None: ...
    def __lt__(self, other: Any) -> Any: ...
    def __le__(self, other: Any) -> Any: ...
    def __gt__(self, other: Any) -> Any: ...
    def __ge__(self, other: Any) -> Any: ...



from .const import ADMIN_SERVICES as ADMIN_SERVICES, ALL_KEYS as ALL_KEYS, CONNECTION_TIMEOUT as CONNECTION_TIMEOUT, DEFAULT_DEVICE_NAME as DEFAULT_DEVICE_NAME, DEFAULT_NOTIFY_SERVICE_NAME as DEFAULT_NOTIFY_SERVICE_NAME, DOMAIN as DOMAIN, KEY_DEVICE_BASIC_INFORMATION as KEY_DEVICE_BASIC_INFORMATION, KEY_DEVICE_INFORMATION as KEY_DEVICE_INFORMATION, KEY_DEVICE_SIGNAL as KEY_DEVICE_SIGNAL, KEY_DIALUP_MOBILE_DATASWITCH as KEY_DIALUP_MOBILE_DATASWITCH, KEY_MONITORING_CHECK_NOTIFICATIONS as KEY_MONITORING_CHECK_NOTIFICATIONS, KEY_MONITORING_MONTH_STATISTICS as KEY_MONITORING_MONTH_STATISTICS, KEY_MONITORING_STATUS as KEY_MONITORING_STATUS, KEY_MONITORING_TRAFFIC_STATISTICS as KEY_MONITORING_TRAFFIC_STATISTICS, KEY_NET_CURRENT_PLMN as KEY_NET_CURRENT_PLMN, KEY_NET_NET_MODE as KEY_NET_NET_MODE, KEY_SMS_SMS_COUNT as KEY_SMS_SMS_COUNT, KEY_WLAN_HOST_LIST as KEY_WLAN_HOST_LIST, KEY_WLAN_WIFI_FEATURE_SWITCH as KEY_WLAN_WIFI_FEATURE_SWITCH, NOTIFY_SUPPRESS_TIMEOUT as NOTIFY_SUPPRESS_TIMEOUT, SERVICE_CLEAR_TRAFFIC_STATISTICS as SERVICE_CLEAR_TRAFFIC_STATISTICS, SERVICE_REBOOT as SERVICE_REBOOT, SERVICE_RESUME_INTEGRATION as SERVICE_RESUME_INTEGRATION, SERVICE_SUSPEND_INTEGRATION as SERVICE_SUSPEND_INTEGRATION, UPDATE_OPTIONS_SIGNAL as UPDATE_OPTIONS_SIGNAL, UPDATE_SIGNAL as UPDATE_SIGNAL
from homeassistant.config_entries import ConfigEntry as ConfigEntry, SOURCE_IMPORT as SOURCE_IMPORT
from homeassistant.const import CONF_NAME as CONF_NAME, CONF_PASSWORD as CONF_PASSWORD, CONF_RECIPIENT as CONF_RECIPIENT, CONF_URL as CONF_URL, CONF_USERNAME as CONF_USERNAME, EVENT_HOMEASSISTANT_STOP as EVENT_HOMEASSISTANT_STOP
from homeassistant.core import CALLBACK_TYPE as CALLBACK_TYPE, ServiceCall as ServiceCall
from homeassistant.exceptions import ConfigEntryNotReady as ConfigEntryNotReady
from homeassistant.helpers import discovery as discovery
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect, async_dispatcher_send as async_dispatcher_send, dispatcher_send as dispatcher_send
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.event import async_track_time_interval as async_track_time_interval
from homeassistant.helpers.typing import ConfigType as ConfigType, HomeAssistantType as HomeAssistantType
from huawei_lte_api.Client import Client
from huawei_lte_api.Connection import Connection
from typing import Any, Callable

_LOGGER: Any
SCAN_INTERVAL: Any
NOTIFY_SCHEMA: Any
CONFIG_SCHEMA: Any
SERVICE_SCHEMA: Any
CONFIG_ENTRY_PLATFORMS: Any

class Router:
    connection: Connection = ...
    url: str = ...
    mac: str = ...
    signal_update: CALLBACK_TYPE = ...
    data: dict[str, Any] = ...
    subscriptions: dict[str, set[str]] = ...
    inflight_gets: set[str] = ...
    unload_handlers: list[CALLBACK_TYPE] = ...
    client: Client
    suspended: Any = ...
    notify_last_attempt: float = ...
    def __attrs_post_init__(self) -> None: ...
    @property
    def device_name(self) -> str: ...
    @property
    def device_identifiers(self) -> set[tuple[str, str]]: ...
    @property
    def device_connections(self) -> set[tuple[str, str]]: ...
    def _get_data(self, key: str, func: Callable[[], Any]) -> None: ...
    def update(self) -> None: ...
    def logout(self) -> None: ...
    def cleanup(self, *_: Any) -> None: ...
    def __init__(self, connection: Any, url: Any, mac: Any, signal_update: Any, data: Any, subscriptions: Any, inflight_gets: Any, unload_handlers: Any, suspended: Any, notify_last_attempt: Any) -> None: ...
    def __lt__(self, other: Any) -> Any: ...
    def __le__(self, other: Any) -> Any: ...
    def __gt__(self, other: Any) -> Any: ...
    def __ge__(self, other: Any) -> Any: ...

class HuaweiLteData:
    hass_config: dict = ...
    config: dict[str, dict[str, Any]] = ...
    routers: dict[str, Router] = ...
    def __init__(self, hass_config: Any, config: Any, routers: Any) -> None: ...
    def __lt__(self, other: Any) -> Any: ...
    def __le__(self, other: Any) -> Any: ...
    def __gt__(self, other: Any) -> Any: ...
    def __ge__(self, other: Any) -> Any: ...

async def async_setup_entry(hass: HomeAssistantType, config_entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistantType, config_entry: ConfigEntry) -> bool: ...
async def async_setup(hass: HomeAssistantType, config: ConfigType) -> bool: ...
async def async_signal_options_update(hass: HomeAssistantType, config_entry: ConfigEntry) -> None: ...
async def async_migrate_entry(hass: HomeAssistantType, config_entry: ConfigEntry) -> bool: ...

class HuaweiLteBaseEntity(Entity):
    router: Router = ...
    _available: bool = ...
    _unsub_handlers: list[Callable] = ...
    @property
    def _entity_name(self) -> str: ...
    @property
    def _device_unique_id(self) -> str: ...
    @property
    def unique_id(self) -> str: ...
    @property
    def name(self) -> str: ...
    @property
    def available(self) -> bool: ...
    @property
    def should_poll(self) -> bool: ...
    @property
    def device_info(self) -> dict[str, Any]: ...
    async def async_update(self) -> None: ...
    async def async_update_options(self, config_entry: ConfigEntry) -> None: ...
    async def async_added_to_hass(self) -> None: ...
    async def _async_maybe_update(self, url: str) -> None: ...
    async def _async_maybe_update_options(self, config_entry: ConfigEntry) -> None: ...
    async def async_will_remove_from_hass(self) -> None: ...
    def __init__(self, router: Any, available: Any, unsub_handlers: Any) -> None: ...
    def __lt__(self, other: Any) -> Any: ...
    def __le__(self, other: Any) -> Any: ...
    def __gt__(self, other: Any) -> Any: ...
    def __ge__(self, other: Any) -> Any: ...



from .const import DATA_CLIENT as DATA_CLIENT, DOMAIN as DOMAIN, EVENT_DEVICE_ADDED_TO_REGISTRY as EVENT_DEVICE_ADDED_TO_REGISTRY
from aiohttp import web
from homeassistant.components import websocket_api as websocket_api
from homeassistant.components.http.view import HomeAssistantView as HomeAssistantView
from homeassistant.components.websocket_api.connection import ActiveConnection as ActiveConnection
from homeassistant.components.websocket_api.const import ERR_NOT_FOUND as ERR_NOT_FOUND, ERR_NOT_SUPPORTED as ERR_NOT_SUPPORTED, ERR_UNKNOWN_ERROR as ERR_UNKNOWN_ERROR
from homeassistant.const import CONF_URL as CONF_URL
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.aiohttp_client import async_get_clientsession as async_get_clientsession
from homeassistant.helpers.device_registry import DeviceEntry as DeviceEntry
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect

ID: str
ENTRY_ID: str
NODE_ID: str
TYPE: str
PROPERTY: str
PROPERTY_KEY: str
VALUE: str
CONFIG: str
LEVEL: str
LOG_TO_FILE: str
FILENAME: str
ENABLED: str
FORCE_CONSOLE: str
VALUE_ID: str
STATUS: str

def async_register_api(hass: HomeAssistant) -> None: ...
def websocket_network_status(hass: HomeAssistant, connection: ActiveConnection, msg: dict) -> None: ...
def websocket_node_status(hass: HomeAssistant, connection: ActiveConnection, msg: dict) -> None: ...
async def websocket_add_node(hass: HomeAssistant, connection: ActiveConnection, msg: dict) -> None: ...
async def websocket_stop_inclusion(hass: HomeAssistant, connection: ActiveConnection, msg: dict) -> None: ...
async def websocket_stop_exclusion(hass: HomeAssistant, connection: ActiveConnection, msg: dict) -> None: ...
async def websocket_remove_node(hass: HomeAssistant, connection: ActiveConnection, msg: dict) -> None: ...
async def websocket_set_config_parameter(hass: HomeAssistant, connection: ActiveConnection, msg: dict) -> None: ...
def websocket_get_config_parameters(hass: HomeAssistant, connection: ActiveConnection, msg: dict) -> None: ...
def filename_is_present_if_logging_to_file(obj: dict) -> dict: ...
async def websocket_update_log_config(hass: HomeAssistant, connection: ActiveConnection, msg: dict) -> None: ...
async def websocket_get_log_config(hass: HomeAssistant, connection: ActiveConnection, msg: dict) -> None: ...

class DumpView(HomeAssistantView):
    url: str = ...
    name: str = ...
    async def get(self, request: web.Request, config_entry_id: str) -> web.Response: ...



from enum import Enum
from homeassistant.components.climate.const import HVAC_MODE_AUTO as HVAC_MODE_AUTO, HVAC_MODE_COOL as HVAC_MODE_COOL, HVAC_MODE_DRY as HVAC_MODE_DRY, HVAC_MODE_FAN_ONLY as HVAC_MODE_FAN_ONLY, HVAC_MODE_HEAT as HVAC_MODE_HEAT, HVAC_MODE_OFF as HVAC_MODE_OFF, PRESET_AWAY as PRESET_AWAY, PRESET_COMFORT as PRESET_COMFORT, PRESET_ECO as PRESET_ECO, PRESET_NONE as PRESET_NONE, PRESET_SLEEP as PRESET_SLEEP
from typing import Any

DOMAIN: str
KNX_ADDRESS: str
CONF_INVERT: str
CONF_STATE_ADDRESS: str
CONF_SYNC_STATE: str
CONF_RESET_AFTER: str
ATTR_COUNTER: str

class ColorTempModes(Enum):
    ABSOLUTE: str = ...
    RELATIVE: str = ...

class SupportedPlatforms(Enum):
    BINARY_SENSOR: str = ...
    CLIMATE: str = ...
    COVER: str = ...
    FAN: str = ...
    LIGHT: str = ...
    NOTIFY: str = ...
    SCENE: str = ...
    SENSOR: str = ...
    SWITCH: str = ...
    WEATHER: str = ...

CONTROLLER_MODES: Any
PRESET_MODES: Any



import abc
from .const import ATTR_MAX as ATTR_MAX, ATTR_MIN as ATTR_MIN, ATTR_STEP as ATTR_STEP, ATTR_VALUE as ATTR_VALUE, DEFAULT_MAX_VALUE as DEFAULT_MAX_VALUE, DEFAULT_MIN_VALUE as DEFAULT_MIN_VALUE, DEFAULT_STEP as DEFAULT_STEP, DOMAIN as DOMAIN, SERVICE_SET_VALUE as SERVICE_SET_VALUE
from abc import abstractmethod
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.helpers.config_validation import PLATFORM_SCHEMA as PLATFORM_SCHEMA, PLATFORM_SCHEMA_BASE as PLATFORM_SCHEMA_BASE
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.entity_component import EntityComponent as EntityComponent
from homeassistant.helpers.typing import ConfigType as ConfigType, HomeAssistantType as HomeAssistantType
from typing import Any

SCAN_INTERVAL: Any
ENTITY_ID_FORMAT: Any
MIN_TIME_BETWEEN_SCANS: Any
_LOGGER: Any

async def async_setup(hass: HomeAssistantType, config: ConfigType) -> bool: ...
async def async_setup_entry(hass: HomeAssistantType, entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistantType, entry: ConfigEntry) -> bool: ...

class NumberEntity(Entity, metaclass=abc.ABCMeta):
    @property
    def capability_attributes(self) -> dict[str, Any]: ...
    @property
    def min_value(self) -> float: ...
    @property
    def max_value(self) -> float: ...
    @property
    def step(self) -> float: ...
    @property
    def state(self) -> float: ...
    @property
    @abstractmethod
    def value(self) -> float: ...
    def set_value(self, value: float) -> None: ...
    async def async_set_value(self, value: float) -> None: ...



from aiohttp import web
from datetime import datetime as dt
from homeassistant.components import recorder as recorder
from homeassistant.components.http import HomeAssistantView as HomeAssistantView
from homeassistant.components.recorder.models import States as States, process_timestamp as process_timestamp, process_timestamp_to_utc_isoformat as process_timestamp_to_utc_isoformat
from homeassistant.components.recorder.util import execute as execute, session_scope as session_scope
from homeassistant.const import CONF_DOMAINS as CONF_DOMAINS, CONF_ENTITIES as CONF_ENTITIES, CONF_EXCLUDE as CONF_EXCLUDE, CONF_INCLUDE as CONF_INCLUDE, HTTP_BAD_REQUEST as HTTP_BAD_REQUEST
from homeassistant.core import Context as Context, HomeAssistant as HomeAssistant, State as State, split_entity_id as split_entity_id
from homeassistant.helpers.entityfilter import CONF_ENTITY_GLOBS as CONF_ENTITY_GLOBS, INCLUDE_EXCLUDE_BASE_FILTER_SCHEMA as INCLUDE_EXCLUDE_BASE_FILTER_SCHEMA
from typing import Any, Iterable, Optional

_LOGGER: Any
DOMAIN: str
CONF_ORDER: str
STATE_KEY: str
LAST_CHANGED_KEY: str
GLOB_TO_SQL_CHARS: Any
CONFIG_SCHEMA: Any
SIGNIFICANT_DOMAINS: Any
IGNORE_DOMAINS: Any
NEED_ATTRIBUTE_DOMAINS: Any
QUERY_STATES: Any
HISTORY_BAKERY: str

def get_significant_states(hass: Any, *args: Any, **kwargs: Any): ...
def _get_significant_states(hass: Any, session: Any, start_time: Any, end_time: Optional[Any] = ..., entity_ids: Optional[Any] = ..., filters: Optional[Any] = ..., include_start_time_state: bool = ..., significant_changes_only: bool = ..., minimal_response: bool = ...): ...
def state_changes_during_period(hass: Any, start_time: Any, end_time: Optional[Any] = ..., entity_id: Optional[Any] = ...): ...
def get_last_state_changes(hass: Any, number_of_states: Any, entity_id: Any): ...
def get_states(hass: Any, utc_point_in_time: Any, entity_ids: Optional[Any] = ..., run: Optional[Any] = ..., filters: Optional[Any] = ...): ...
def _get_states_with_session(hass: Any, session: Any, utc_point_in_time: Any, entity_ids: Optional[Any] = ..., run: Optional[Any] = ..., filters: Optional[Any] = ...): ...
def _get_single_entity_states_with_session(hass: Any, session: Any, utc_point_in_time: Any, entity_id: Any): ...
def _sorted_states_to_json(hass: Any, session: Any, states: Any, start_time: Any, entity_ids: Any, filters: Optional[Any] = ..., include_start_time_state: bool = ..., minimal_response: bool = ...): ...
def get_state(hass: Any, utc_point_in_time: Any, entity_id: Any, run: Optional[Any] = ...): ...
async def async_setup(hass: Any, config: Any): ...

class HistoryPeriodView(HomeAssistantView):
    url: str = ...
    name: str = ...
    extra_urls: Any = ...
    filters: Any = ...
    use_include_order: Any = ...
    def __init__(self, filters: Any, use_include_order: Any) -> None: ...
    async def get(self, request: web.Request, datetime: Union[str, None]=...) -> web.Response: ...
    def _sorted_significant_states_json(self, hass: Any, start_time: Any, end_time: Any, entity_ids: Any, include_start_time_state: Any, significant_changes_only: Any, minimal_response: Any): ...

def sqlalchemy_filter_from_include_exclude_conf(conf: Any): ...

class Filters:
    excluded_entities: Any = ...
    excluded_domains: Any = ...
    excluded_entity_globs: Any = ...
    included_entities: Any = ...
    included_domains: Any = ...
    included_entity_globs: Any = ...
    def __init__(self) -> None: ...
    def apply(self, query: Any): ...
    @property
    def has_config(self): ...
    def bake(self, baked_query: Any): ...
    def entity_filter(self): ...

def _glob_to_like(glob_str: Any): ...
def _entities_may_have_state_changes_after(hass: HomeAssistant, entity_ids: Iterable, start_time: dt) -> bool: ...

class LazyState(State):
    __slots__: Any = ...
    _row: Any = ...
    entity_id: Any = ...
    state: Any = ...
    _attributes: Any = ...
    _last_changed: Any = ...
    _last_updated: Any = ...
    _context: Any = ...
    def __init__(self, row: Any) -> None: ...
    @property
    def attributes(self): ...
    @attributes.setter
    def attributes(self, value: Any) -> None: ...
    @property
    def context(self): ...
    @context.setter
    def context(self, value: Any) -> None: ...
    @property
    def last_changed(self): ...
    @last_changed.setter
    def last_changed(self, value: Any) -> None: ...
    @property
    def last_updated(self): ...
    @last_updated.setter
    def last_updated(self, value: Any) -> None: ...
    def as_dict(self): ...
    def __eq__(self, other: Any) -> Any: ...



from .auth import setup_auth as setup_auth
from .ban import setup_bans as setup_bans
from .const import KEY_AUTHENTICATED as KEY_AUTHENTICATED, KEY_HASS as KEY_HASS, KEY_HASS_USER as KEY_HASS_USER
from .cors import setup_cors as setup_cors
from .forwarded import async_setup_forwarded as async_setup_forwarded
from .request_context import setup_request_context as setup_request_context
from .security_filter import setup_security_filter as setup_security_filter
from .static import CACHE_HEADERS as CACHE_HEADERS, CachingStaticResource as CachingStaticResource
from .view import HomeAssistantView as HomeAssistantView
from .web_runner import HomeAssistantTCPSite as HomeAssistantTCPSite
from aiohttp import web
from contextvars import ContextVar
from homeassistant.const import EVENT_HOMEASSISTANT_START as EVENT_HOMEASSISTANT_START, EVENT_HOMEASSISTANT_STOP as EVENT_HOMEASSISTANT_STOP, SERVER_PORT as SERVER_PORT
from homeassistant.core import Event as Event, HomeAssistant as HomeAssistant
from homeassistant.helpers import storage as storage
from homeassistant.loader import bind_hass as bind_hass
from homeassistant.setup import ATTR_COMPONENT as ATTR_COMPONENT, EVENT_COMPONENT_LOADED as EVENT_COMPONENT_LOADED
from typing import Any

DOMAIN: str
CONF_SERVER_HOST: str
CONF_SERVER_PORT: str
CONF_BASE_URL: str
CONF_SSL_CERTIFICATE: str
CONF_SSL_PEER_CERTIFICATE: str
CONF_SSL_KEY: str
CONF_CORS_ORIGINS: str
CONF_USE_X_FORWARDED_FOR: str
CONF_TRUSTED_PROXIES: str
CONF_LOGIN_ATTEMPTS_THRESHOLD: str
CONF_IP_BAN_ENABLED: str
CONF_SSL_PROFILE: str
SSL_MODERN: str
SSL_INTERMEDIATE: str
_LOGGER: Any
DEFAULT_DEVELOPMENT: str
DEFAULT_CORS: Any
NO_LOGIN_ATTEMPT_THRESHOLD: int
MAX_CLIENT_SIZE: int
STORAGE_KEY = DOMAIN
STORAGE_VERSION: int
HTTP_SCHEMA: Any
CONFIG_SCHEMA: Any

async def async_get_last_config(hass: HomeAssistant) -> Union[dict, None]: ...

class ApiConfig:
    local_ip: Any = ...
    host: Any = ...
    port: Any = ...
    use_ssl: Any = ...
    def __init__(self, local_ip: str, host: str, port: Union[int, None]=..., use_ssl: bool=...) -> None: ...

async def async_setup(hass: Any, config: Any): ...

class HomeAssistantHTTP:
    hass: Any = ...
    ssl_certificate: Any = ...
    ssl_peer_certificate: Any = ...
    ssl_key: Any = ...
    server_host: Any = ...
    server_port: Any = ...
    trusted_proxies: Any = ...
    is_ban_enabled: Any = ...
    ssl_profile: Any = ...
    _handler: Any = ...
    runner: Any = ...
    site: Any = ...
    def __init__(self, hass: Any, ssl_certificate: Any, ssl_peer_certificate: Any, ssl_key: Any, server_host: Any, server_port: Any, cors_origins: Any, use_x_forwarded_for: Any, trusted_proxies: Any, login_threshold: Any, is_ban_enabled: Any, ssl_profile: Any) -> None: ...
    def register_view(self, view: Any) -> None: ...
    def register_redirect(self, url: Any, redirect_to: Any, *, redirect_exc: Any = ...) -> None: ...
    def register_static_path(self, url_path: Any, path: Any, cache_headers: bool = ...): ...
    async def start(self) -> None: ...
    async def stop(self) -> None: ...

async def start_http_server_and_save_config(hass: HomeAssistant, conf: dict, server: HomeAssistantHTTP) -> None: ...

current_request: ContextVar[Union[web.Request, None]]



from . import ATTR_VALUE as ATTR_VALUE, DOMAIN as DOMAIN, SERVICE_SET_VALUE as SERVICE_SET_VALUE
from homeassistant.const import ATTR_ENTITY_ID as ATTR_ENTITY_ID
from homeassistant.core import Context as Context, State as State
from homeassistant.helpers.typing import HomeAssistantType as HomeAssistantType
from typing import Any, Iterable

_LOGGER: Any

async def _async_reproduce_state(hass: HomeAssistantType, state: State, *, context: Union[Context, None]=..., reproduce_options: Union[dict[str, Any], None]=...) -> None: ...
async def async_reproduce_states(hass: HomeAssistantType, states: Iterable[State], *, context: Union[Context, None]=..., reproduce_options: Union[dict[str, Any], None]=...) -> None: ...



from .const import CONTROLLER_MODES as CONTROLLER_MODES, DOMAIN as DOMAIN, PRESET_MODES as PRESET_MODES
from .knx_entity import KnxEntity as KnxEntity
from homeassistant.components.climate import ClimateEntity as ClimateEntity
from homeassistant.components.climate.const import HVAC_MODE_HEAT as HVAC_MODE_HEAT, HVAC_MODE_OFF as HVAC_MODE_OFF, PRESET_AWAY as PRESET_AWAY, SUPPORT_PRESET_MODE as SUPPORT_PRESET_MODE, SUPPORT_TARGET_TEMPERATURE as SUPPORT_TARGET_TEMPERATURE
from homeassistant.const import ATTR_TEMPERATURE as ATTR_TEMPERATURE, TEMP_CELSIUS as TEMP_CELSIUS
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.typing import ConfigType as ConfigType, DiscoveryInfoType as DiscoveryInfoType
from typing import Any, Callable, Iterable
from xknx.devices import Climate as XknxClimate

CONTROLLER_MODES_INV: Any
PRESET_MODES_INV: Any

async def async_setup_platform(hass: HomeAssistant, config: ConfigType, async_add_entities: Callable[[Iterable[Entity]], None], discovery_info: Union[DiscoveryInfoType, None]=...) -> None: ...

class KNXClimate(KnxEntity, ClimateEntity):
    _device: Any
    _unit_of_measurement: Any = ...
    def __init__(self, device: XknxClimate) -> None: ...
    @property
    def supported_features(self) -> int: ...
    async def async_update(self) -> None: ...
    @property
    def temperature_unit(self) -> str: ...
    @property
    def current_temperature(self) -> Union[float, None]: ...
    @property
    def target_temperature_step(self) -> float: ...
    @property
    def target_temperature(self) -> Union[float, None]: ...
    @property
    def min_temp(self) -> float: ...
    @property
    def max_temp(self) -> float: ...
    async def async_set_temperature(self, **kwargs: Any) -> None: ...
    @property
    def hvac_mode(self) -> str: ...
    @property
    def hvac_modes(self) -> list[str]: ...
    async def async_set_hvac_mode(self, hvac_mode: str) -> None: ...
    @property
    def preset_mode(self) -> Union[str, None]: ...
    @property
    def preset_modes(self) -> Union[list[str], None]: ...
    async def async_set_preset_mode(self, preset_mode: str) -> None: ...



from . import Router as Router
from .const import DOMAIN as DOMAIN
from homeassistant.components.notify import ATTR_TARGET as ATTR_TARGET, BaseNotificationService as BaseNotificationService
from homeassistant.const import CONF_RECIPIENT as CONF_RECIPIENT, CONF_URL as CONF_URL
from homeassistant.helpers.typing import HomeAssistantType as HomeAssistantType
from typing import Any

_LOGGER: Any

async def async_get_service(hass: HomeAssistantType, config: dict[str, Any], discovery_info: Union[dict[str, Any], None]=...) -> Union[HuaweiLteSmsNotificationService, None]: ...

class HuaweiLteSmsNotificationService(BaseNotificationService):
    router: Router = ...
    default_targets: list[str] = ...
    def send_message(self, message: str=..., **kwargs: Any) -> None: ...
    def __init__(self, router: Any, default_targets: Any) -> None: ...
    def __lt__(self, other: Any) -> Any: ...
    def __le__(self, other: Any) -> Any: ...
    def __gt__(self, other: Any) -> Any: ...
    def __ge__(self, other: Any) -> Any: ...



from . import AUTH_PROVIDERS as AUTH_PROVIDERS, AUTH_PROVIDER_SCHEMA as AUTH_PROVIDER_SCHEMA, AuthProvider as AuthProvider, LoginFlow as LoginFlow
from ..models import Credentials as Credentials, UserMeta as UserMeta
from homeassistant.const import CONF_COMMAND as CONF_COMMAND
from homeassistant.exceptions import HomeAssistantError as HomeAssistantError
from typing import Any

CONF_ARGS: str
CONF_META: str
CONFIG_SCHEMA: Any
_LOGGER: Any

class InvalidAuthError(HomeAssistantError): ...

class CommandLineAuthProvider(AuthProvider):
    DEFAULT_TITLE: str = ...
    ALLOWED_META_KEYS: Any = ...
    _user_meta: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    async def async_login_flow(self, context: Union[dict, None]) -> LoginFlow: ...
    async def async_validate_login(self, username: str, password: str) -> None: ...
    async def async_get_or_create_credentials(self, flow_result: dict[str, str]) -> Credentials: ...
    async def async_user_meta_for_credentials(self, credentials: Credentials) -> UserMeta: ...

class CommandLineLoginFlow(LoginFlow):
    async def async_step_init(self, user_input: Union[dict[str, str], None]=...) -> dict[str, Any]: ...



from homeassistant.components import websocket_api as websocket_api
from typing import Any

DATA_STORAGE: str
STORAGE_VERSION_USER_DATA: int

async def async_setup_frontend_storage(hass: Any) -> None: ...
def with_store(orig_func: Any): ...
async def websocket_set_user_data(hass: Any, connection: Any, msg: Any, store: Any, data: Any) -> None: ...
async def websocket_get_user_data(hass: Any, connection: Any, msg: Any, store: Any, data: Any) -> None: ...



from homeassistant import data_entry_flow as data_entry_flow, loader as loader
from homeassistant.const import EVENT_HOMEASSISTANT_STARTED as EVENT_HOMEASSISTANT_STARTED
from homeassistant.core import CALLBACK_TYPE as CALLBACK_TYPE, CoreState as CoreState, HomeAssistant as HomeAssistant, callback as callback
from homeassistant.exceptions import ConfigEntryNotReady as ConfigEntryNotReady, HomeAssistantError as HomeAssistantError
from homeassistant.helpers import device_registry as device_registry, entity_registry as entity_registry
from homeassistant.helpers.event import Event as Event
from homeassistant.helpers.typing import UNDEFINED as UNDEFINED, UndefinedType as UndefinedType
from homeassistant.setup import async_process_deps_reqs as async_process_deps_reqs, async_setup_component as async_setup_component
from homeassistant.util.decorator import Registry as Registry
from typing import Any

_LOGGER: Any
SOURCE_DISCOVERY: str
SOURCE_HASSIO: str
SOURCE_HOMEKIT: str
SOURCE_IMPORT: str
SOURCE_INTEGRATION_DISCOVERY: str
SOURCE_MQTT: str
SOURCE_SSDP: str
SOURCE_USER: str
SOURCE_ZEROCONF: str
SOURCE_DHCP: str
SOURCE_IGNORE: str
SOURCE_UNIGNORE: str
SOURCE_REAUTH: str
HANDLERS: Any
STORAGE_KEY: str
STORAGE_VERSION: int
PATH_CONFIG: str
SAVE_DELAY: int
ENTRY_STATE_LOADED: str
ENTRY_STATE_SETUP_ERROR: str
ENTRY_STATE_MIGRATION_ERROR: str
ENTRY_STATE_SETUP_RETRY: str
ENTRY_STATE_NOT_LOADED: str
ENTRY_STATE_FAILED_UNLOAD: str
UNRECOVERABLE_STATES: Any
DEFAULT_DISCOVERY_UNIQUE_ID: str
DISCOVERY_NOTIFICATION_ID: str
DISCOVERY_SOURCES: Any
RECONFIGURE_NOTIFICATION_ID: str
EVENT_FLOW_DISCOVERED: str
CONN_CLASS_CLOUD_PUSH: str
CONN_CLASS_CLOUD_POLL: str
CONN_CLASS_LOCAL_PUSH: str
CONN_CLASS_LOCAL_POLL: str
CONN_CLASS_ASSUMED: str
CONN_CLASS_UNKNOWN: str
DISABLED_USER: str
RELOAD_AFTER_UPDATE_DELAY: int

class ConfigError(HomeAssistantError): ...
class UnknownEntry(ConfigError): ...
class OperationNotAllowed(ConfigError): ...

UpdateListenerType: Any

class ConfigEntry:
    __slots__: Any = ...
    entry_id: Any = ...
    version: Any = ...
    domain: Any = ...
    title: Any = ...
    data: Any = ...
    options: Any = ...
    system_options: Any = ...
    source: Any = ...
    connection_class: Any = ...
    state: Any = ...
    unique_id: Any = ...
    disabled_by: Any = ...
    supports_unload: bool = ...
    update_listeners: Any = ...
    _async_cancel_retry_setup: Any = ...
    def __init__(self, version: int, domain: str, title: str, data: dict, source: str, connection_class: str, system_options: dict, options: Union[dict, None]=..., unique_id: Union[str, None]=..., entry_id: Union[str, None]=..., state: str=..., disabled_by: Union[str, None]=...) -> None: ...
    async def async_setup(self, hass: HomeAssistant, *, integration: Union[loader.Integration, None]=..., tries: int=...) -> None: ...
    async def async_unload(self, hass: HomeAssistant, *, integration: Union[loader.Integration, None]=...) -> bool: ...
    async def async_remove(self, hass: HomeAssistant) -> None: ...
    async def async_migrate(self, hass: HomeAssistant) -> bool: ...
    def add_update_listener(self, listener: UpdateListenerType) -> CALLBACK_TYPE: ...
    def as_dict(self) -> dict[str, Any]: ...

class ConfigEntriesFlowManager(data_entry_flow.FlowManager):
    config_entries: Any = ...
    _hass_config: Any = ...
    def __init__(self, hass: HomeAssistant, config_entries: ConfigEntries, hass_config: dict) -> None: ...
    async def async_finish_flow(self, flow: data_entry_flow.FlowHandler, result: dict[str, Any]) -> dict[str, Any]: ...
    async def async_create_flow(self, handler_key: Any, *, context: Union[dict, None]=..., data: Any=...) -> ConfigFlow: ...
    async def async_post_init(self, flow: data_entry_flow.FlowHandler, result: dict) -> None: ...

class ConfigEntries:
    hass: Any = ...
    flow: Any = ...
    options: Any = ...
    _hass_config: Any = ...
    _entries: Any = ...
    _store: Any = ...
    def __init__(self, hass: HomeAssistant, hass_config: dict) -> None: ...
    def async_domains(self, include_ignore: bool=..., include_disabled: bool=...) -> list[str]: ...
    def async_get_entry(self, entry_id: str) -> Union[ConfigEntry, None]: ...
    def async_entries(self, domain: Union[str, None]=...) -> list[ConfigEntry]: ...
    async def async_add(self, entry: ConfigEntry) -> None: ...
    async def async_remove(self, entry_id: str) -> dict[str, Any]: ...
    async def async_initialize(self) -> None: ...
    async def async_setup(self, entry_id: str) -> bool: ...
    async def async_unload(self, entry_id: str) -> bool: ...
    async def async_reload(self, entry_id: str) -> bool: ...
    async def async_set_disabled_by(self, entry_id: str, disabled_by: Union[str, None]) -> bool: ...
    def async_update_entry(self, entry: ConfigEntry, *, unique_id: Union[str, dict, None, UndefinedType]=..., title: Union[str, dict, UndefinedType]=..., data: Union[dict, UndefinedType]=..., options: Union[dict, UndefinedType]=..., system_options: Union[dict, UndefinedType]=...) -> bool: ...
    async def async_forward_entry_setup(self, entry: ConfigEntry, domain: str) -> bool: ...
    async def async_forward_entry_unload(self, entry: ConfigEntry, domain: str) -> bool: ...
    def _async_schedule_save(self) -> None: ...
    def _data_to_save(self) -> dict[str, list[dict[str, Any]]]: ...

async def _old_conf_migrator(old_config: dict[str, Any]) -> dict[str, Any]: ...

class ConfigFlow(data_entry_flow.FlowHandler):
    def __init_subclass__(cls: Any, domain: Union[str, None]=..., **kwargs: Any) -> None: ...
    CONNECTION_CLASS: Any = ...
    @property
    def unique_id(self) -> Union[str, None]: ...
    @staticmethod
    def async_get_options_flow(config_entry: ConfigEntry) -> OptionsFlow: ...
    def _abort_if_unique_id_configured(self, updates: Union[dict[Any, Any], None]=..., reload_on_update: bool=...) -> None: ...
    async def async_set_unique_id(self, unique_id: Union[str, None]=..., *, raise_on_progress: bool=...) -> Union[ConfigEntry, None]: ...
    def _set_confirm_only(self) -> None: ...
    def _async_current_entries(self, include_ignore: Union[bool, None]=...) -> list[ConfigEntry]: ...
    def _async_current_ids(self, include_ignore: bool=...) -> set[Union[str, None]]: ...
    def _async_in_progress(self, include_uninitialized: bool=...) -> list[dict]: ...
    async def async_step_ignore(self, user_input: dict[str, Any]) -> dict[str, Any]: ...
    async def async_step_unignore(self, user_input: dict[str, Any]) -> dict[str, Any]: ...
    async def async_step_user(self, user_input: Union[dict[str, Any], None]=...) -> dict[str, Any]: ...
    async def _async_handle_discovery_without_unique_id(self) -> None: ...
    async def async_step_discovery(self, discovery_info: dict[str, Any]) -> dict[str, Any]: ...
    def async_abort(self, reason: str, *, description_placeholders: Union[dict, None]=...) -> dict[str, Any]: ...
    async_step_hassio: Any = ...
    async_step_homekit: Any = ...
    async_step_mqtt: Any = ...
    async_step_ssdp: Any = ...
    async_step_zeroconf: Any = ...
    async_step_dhcp: Any = ...

class OptionsFlowManager(data_entry_flow.FlowManager):
    async def async_create_flow(self, handler_key: Any, *, context: Union[dict[str, Any], None]=..., data: Union[dict[str, Any], None]=...) -> OptionsFlow: ...
    async def async_finish_flow(self, flow: data_entry_flow.FlowHandler, result: dict[str, Any]) -> dict[str, Any]: ...

class OptionsFlow(data_entry_flow.FlowHandler):
    handler: str

class SystemOptions:
    disable_new_entities: bool = ...
    def update(self, disable_new_entities: bool) -> None: ...
    def as_dict(self) -> dict[str, Any]: ...
    def __init__(self, disable_new_entities: Any) -> None: ...
    def __lt__(self, other: Any) -> Any: ...
    def __le__(self, other: Any) -> Any: ...
    def __gt__(self, other: Any) -> Any: ...
    def __ge__(self, other: Any) -> Any: ...

class EntityRegistryDisabledHandler:
    hass: Any = ...
    registry: Any = ...
    changed: Any = ...
    _remove_call_later: Any = ...
    def __init__(self, hass: HomeAssistant) -> None: ...
    def async_setup(self) -> None: ...
    async def _handle_entry_updated(self, event: Event) -> None: ...
    async def _handle_reload(self, _now: Any) -> None: ...

def _handle_entry_updated_filter(event: Event) -> bool: ...
async def support_entry_unload(hass: HomeAssistant, domain: str) -> bool: ...



import asyncio
from homeassistant import bootstrap as bootstrap
from homeassistant.core import callback as callback
from homeassistant.helpers.frame import warn_use as warn_use
from typing import Any

MAX_EXECUTOR_WORKERS: int

class RuntimeConfig:
    config_dir: str
    skip_pip: bool = ...
    safe_mode: bool = ...
    verbose: bool = ...
    log_rotate_days: Union[int, None] = ...
    log_file: Union[str, None] = ...
    log_no_color: bool = ...
    debug: bool = ...
    open_ui: bool = ...
    def __init__(self, config_dir: Any, skip_pip: Any, safe_mode: Any, verbose: Any, log_rotate_days: Any, log_file: Any, log_no_color: Any, debug: Any, open_ui: Any) -> None: ...

class HassEventLoopPolicy(asyncio.DefaultEventLoopPolicy):
    debug: Any = ...
    def __init__(self, debug: bool) -> None: ...
    @property
    def loop_name(self) -> str: ...
    def new_event_loop(self) -> asyncio.AbstractEventLoop: ...

def _async_loop_exception_handler(_: Any, context: dict[str, Any]) -> None: ...
async def setup_and_run_hass(runtime_config: RuntimeConfig) -> int: ...
def run(runtime_config: RuntimeConfig) -> int: ...



from .const import DOMAIN as DOMAIN
from .knx_entity import KnxEntity as KnxEntity
from homeassistant.components.switch import SwitchEntity as SwitchEntity
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.typing import ConfigType as ConfigType, DiscoveryInfoType as DiscoveryInfoType
from typing import Any, Callable, Iterable
from xknx.devices import Switch as XknxSwitch

async def async_setup_platform(hass: HomeAssistant, config: ConfigType, async_add_entities: Callable[[Iterable[Entity]], None], discovery_info: Union[DiscoveryInfoType, None]=...) -> None: ...

class KNXSwitch(KnxEntity, SwitchEntity):
    _device: Any
    def __init__(self, device: XknxSwitch) -> None: ...
    @property
    def is_on(self) -> bool: ...
    async def async_turn_on(self, **kwargs: Any) -> None: ...
    async def async_turn_off(self, **kwargs: Any) -> None: ...



from .const import ColorTempModes as ColorTempModes, KNX_ADDRESS as KNX_ADDRESS, SupportedPlatforms as SupportedPlatforms
from .schema import BinarySensorSchema as BinarySensorSchema, ClimateSchema as ClimateSchema, CoverSchema as CoverSchema, FanSchema as FanSchema, LightSchema as LightSchema, SceneSchema as SceneSchema, SensorSchema as SensorSchema, SwitchSchema as SwitchSchema, WeatherSchema as WeatherSchema
from homeassistant.const import CONF_DEVICE_CLASS as CONF_DEVICE_CLASS, CONF_NAME as CONF_NAME, CONF_TYPE as CONF_TYPE
from homeassistant.helpers.typing import ConfigType as ConfigType
from xknx import XKNX as XKNX
from xknx.devices import BinarySensor as XknxBinarySensor, Climate as XknxClimate, Cover as XknxCover, Device as XknxDevice, Fan as XknxFan, Light as XknxLight, Notification as XknxNotification, Scene as XknxScene, Sensor as XknxSensor, Switch as XknxSwitch, Weather as XknxWeather

def create_knx_device(platform: SupportedPlatforms, knx_module: XKNX, config: ConfigType) -> XknxDevice: ...
def _create_cover(knx_module: XKNX, config: ConfigType) -> XknxCover: ...
def _create_light_color(color: str, config: ConfigType) -> tuple[Union[str, None], Union[str, None], Union[str, None], Union[str, None]]: ...
def _create_light(knx_module: XKNX, config: ConfigType) -> XknxLight: ...
def _create_climate(knx_module: XKNX, config: ConfigType) -> XknxClimate: ...
def _create_switch(knx_module: XKNX, config: ConfigType) -> XknxSwitch: ...
def _create_sensor(knx_module: XKNX, config: ConfigType) -> XknxSensor: ...
def _create_notify(knx_module: XKNX, config: ConfigType) -> XknxNotification: ...
def _create_scene(knx_module: XKNX, config: ConfigType) -> XknxScene: ...
def _create_binary_sensor(knx_module: XKNX, config: ConfigType) -> XknxBinarySensor: ...
def _create_weather(knx_module: XKNX, config: ConfigType) -> XknxWeather: ...
def _create_fan(knx_module: XKNX, config: ConfigType) -> XknxFan: ...



import aiohttp
from collections import namedtuple
from typing import Any

ELEVATION_URL: str
IP_API: str
IPAPI: str
AXIS_A: int
FLATTENING: Any
AXIS_B: float
MILES_PER_KILOMETER: float
MAX_ITERATIONS: int
CONVERGENCE_THRESHOLD: float

LocationInfo = namedtuple('LocationInfo', ['ip', 'country_code', 'country_name', 'region_code', 'region_name', 'city', 'zip_code', 'time_zone', 'latitude', 'longitude', 'use_metric'])

async def async_detect_location_info(session: aiohttp.ClientSession) -> Union[LocationInfo, None]: ...
def distance(lat1: Union[float, None], lon1: Union[float, None], lat2: float, lon2: float) -> Union[float, None]: ...
def vincenty(point1: tuple[float, float], point2: tuple[float, float], miles: bool=...) -> Union[float, None]: ...
async def _get_ipapi(session: aiohttp.ClientSession) -> Union[dict[str, Any], None]: ...
async def _get_ip_api(session: aiohttp.ClientSession) -> Union[dict[str, Any], None]: ...



from homeassistant.components.group import GroupIntegrationRegistry as GroupIntegrationRegistry
from homeassistant.const import STATE_LOCKED as STATE_LOCKED, STATE_UNLOCKED as STATE_UNLOCKED
from homeassistant.core import callback as callback
from homeassistant.helpers.typing import HomeAssistantType as HomeAssistantType

def async_describe_on_off_states(hass: HomeAssistantType, registry: GroupIntegrationRegistry) -> None: ...



from .const import DOMAIN as DOMAIN
from .discovery import ZwaveDiscoveryInfo as ZwaveDiscoveryInfo
from .helpers import get_unique_id as get_unique_id
from homeassistant.core import callback as callback
from homeassistant.helpers.entity_registry import EntityRegistry as EntityRegistry
from typing import Any
from zwave_js_server.client import Client as ZwaveClient
from zwave_js_server.model.value import Value as ZwaveValue

_LOGGER: Any

def async_migrate_entity(ent_reg: EntityRegistry, platform: str, old_unique_id: str, new_unique_id: str) -> None: ...
def async_migrate_discovered_value(ent_reg: EntityRegistry, client: ZwaveClient, disc_info: ZwaveDiscoveryInfo) -> None: ...
def get_old_value_ids(value: ZwaveValue) -> list[str]: ...



from datetime import datetime
from homeassistant.config import load_yaml_config_file as load_yaml_config_file
from homeassistant.const import HTTP_BAD_REQUEST as HTTP_BAD_REQUEST
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.exceptions import HomeAssistantError as HomeAssistantError
from homeassistant.util import yaml as yaml
from typing import Any

_LOGGER: Any
KEY_BANNED_IPS: str
KEY_FAILED_LOGIN_ATTEMPTS: str
KEY_LOGIN_THRESHOLD: str
NOTIFICATION_ID_BAN: str
NOTIFICATION_ID_LOGIN: str
IP_BANS_FILE: str
ATTR_BANNED_AT: str
SCHEMA_IP_BAN_ENTRY: Any

def setup_bans(hass: Any, app: Any, login_threshold: Any) -> None: ...
async def ban_middleware(request: Any, handler: Any): ...
def log_invalid_auth(func: Any): ...
async def process_wrong_login(request: Any) -> None: ...
async def process_success_login(request: Any) -> None: ...

class IpBan:
    ip_address: Any = ...
    banned_at: Any = ...
    def __init__(self, ip_ban: str, banned_at: Union[datetime, None]=...) -> None: ...

async def async_load_ip_bans_config(hass: HomeAssistant, path: str) -> list[IpBan]: ...
def update_ip_bans_config(path: str, ip_ban: IpBan) -> None: ...



from PIL import ImageDraw as ImageDraw

def draw_box(draw: ImageDraw, box: tuple[float, float, float, float], img_width: int, img_height: int, text: str=..., color: tuple[int, int, int]=...) -> None: ...



from typing import Any
from zeroconf import DNSRecord as DNSRecord, ServiceBrowser, Zeroconf

class HaZeroconf(Zeroconf):
    def close(self) -> None: ...
    ha_close: Any = ...

class HaServiceBrowser(ServiceBrowser):
    def update_record(self, zc: Zeroconf, now: float, record: DNSRecord) -> None: ...



from homeassistant.const import PRESSURE as PRESSURE, PRESSURE_HPA as PRESSURE_HPA, PRESSURE_INHG as PRESSURE_INHG, PRESSURE_MBAR as PRESSURE_MBAR, PRESSURE_PA as PRESSURE_PA, PRESSURE_PSI as PRESSURE_PSI, UNIT_NOT_RECOGNIZED_TEMPLATE as UNIT_NOT_RECOGNIZED_TEMPLATE
from typing import Any

VALID_UNITS: Any
UNIT_CONVERSION: Any

def convert(value: float, unit_1: str, unit_2: str) -> float: ...



from .core import Context as Context
from typing import Any, Generator, Sequence

class HomeAssistantError(Exception): ...
class InvalidEntityFormatError(HomeAssistantError): ...
class NoEntitySpecifiedError(HomeAssistantError): ...

class TemplateError(HomeAssistantError):
    def __init__(self, exception: Exception) -> None: ...

class ConditionError(HomeAssistantError):
    type: str = ...
    @staticmethod
    def _indent(indent: int, message: str) -> str: ...
    def output(self, indent: int) -> Generator: ...
    def __str__(self) -> str: ...
    def __init__(self, type: Any) -> None: ...
    def __lt__(self, other: Any) -> Any: ...
    def __le__(self, other: Any) -> Any: ...
    def __gt__(self, other: Any) -> Any: ...
    def __ge__(self, other: Any) -> Any: ...

class ConditionErrorMessage(ConditionError):
    message: str = ...
    def output(self, indent: int) -> Generator: ...
    def __init__(self, type: Any, message: Any) -> None: ...
    def __lt__(self, other: Any) -> Any: ...
    def __le__(self, other: Any) -> Any: ...
    def __gt__(self, other: Any) -> Any: ...
    def __ge__(self, other: Any) -> Any: ...

class ConditionErrorIndex(ConditionError):
    index: int = ...
    total: int = ...
    error: ConditionError = ...
    def output(self, indent: int) -> Generator: ...
    def __init__(self, type: Any, index: Any, total: Any, error: Any) -> None: ...
    def __lt__(self, other: Any) -> Any: ...
    def __le__(self, other: Any) -> Any: ...
    def __gt__(self, other: Any) -> Any: ...
    def __ge__(self, other: Any) -> Any: ...

class ConditionErrorContainer(ConditionError):
    errors: Sequence[ConditionError] = ...
    def output(self, indent: int) -> Generator: ...
    def __init__(self, type: Any, errors: Any) -> None: ...
    def __lt__(self, other: Any) -> Any: ...
    def __le__(self, other: Any) -> Any: ...
    def __gt__(self, other: Any) -> Any: ...
    def __ge__(self, other: Any) -> Any: ...

class PlatformNotReady(HomeAssistantError): ...
class ConfigEntryNotReady(HomeAssistantError): ...
class InvalidStateError(HomeAssistantError): ...

class Unauthorized(HomeAssistantError):
    context: Any = ...
    user_id: Any = ...
    entity_id: Any = ...
    config_entry_id: Any = ...
    perm_category: Any = ...
    permission: Any = ...
    def __init__(self, context: Union[Context, None]=..., user_id: Union[str, None]=..., entity_id: Union[str, None]=..., config_entry_id: Union[str, None]=..., perm_category: Union[str, None]=..., permission: Union[str, None]=...) -> None: ...

class UnknownUser(Unauthorized): ...

class ServiceNotFound(HomeAssistantError):
    domain: Any = ...
    service: Any = ...
    def __init__(self, domain: str, service: str) -> None: ...
    def __str__(self) -> str: ...



from homeassistant.const import HTTP_HEADER_X_REQUESTED_WITH as HTTP_HEADER_X_REQUESTED_WITH
from homeassistant.core import callback as callback
from typing import Any

ALLOWED_CORS_HEADERS: Any
VALID_CORS_TYPES: Any

def setup_cors(app: Any, origins: Any): ...



from .const import DATA_CLIENT as DATA_CLIENT, DOMAIN as DOMAIN
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from zwave_js_server.client import Client as ZwaveClient
from zwave_js_server.model.node import Node as ZwaveNode

def get_unique_id(home_id: str, value_id: str) -> str: ...
def get_device_id(client: ZwaveClient, node: ZwaveNode) -> tuple[str, str]: ...
def get_home_and_node_id_from_device_id(device_id: tuple[str, str]) -> list[str]: ...
def async_get_node_from_device_id(hass: HomeAssistant, device_id: str) -> ZwaveNode: ...
def async_get_node_from_entity_id(hass: HomeAssistant, entity_id: str) -> ZwaveNode: ...



from . import const as const, messages as messages
from homeassistant.core import Context as Context, callback as callback
from homeassistant.exceptions import HomeAssistantError as HomeAssistantError, Unauthorized as Unauthorized
from typing import Any

class ActiveConnection:
    logger: Any = ...
    hass: Any = ...
    send_message: Any = ...
    user: Any = ...
    refresh_token_id: Any = ...
    subscriptions: Any = ...
    last_id: int = ...
    def __init__(self, logger: Any, hass: Any, send_message: Any, user: Any, refresh_token: Any) -> None: ...
    def context(self, msg: Any): ...
    def send_result(self, msg_id: int, result: Union[Any, None]=...) -> None: ...
    async def send_big_result(self, msg_id: Any, result: Any) -> None: ...
    def send_error(self, msg_id: int, code: str, message: str) -> None: ...
    def async_handle(self, msg: Any) -> None: ...
    def async_close(self) -> None: ...
    def async_handle_exception(self, msg: Any, err: Any) -> None: ...



from .const import DATA_CLIENT as DATA_CLIENT, DATA_UNSUBSCRIBE as DATA_UNSUBSCRIBE, DOMAIN as DOMAIN
from .discovery import ZwaveDiscoveryInfo as ZwaveDiscoveryInfo
from .entity import ZWaveBaseEntity as ZWaveBaseEntity
from homeassistant.components.light import ATTR_BRIGHTNESS as ATTR_BRIGHTNESS, ATTR_COLOR_TEMP as ATTR_COLOR_TEMP, ATTR_HS_COLOR as ATTR_HS_COLOR, ATTR_TRANSITION as ATTR_TRANSITION, ATTR_WHITE_VALUE as ATTR_WHITE_VALUE, LightEntity as LightEntity, SUPPORT_BRIGHTNESS as SUPPORT_BRIGHTNESS, SUPPORT_COLOR as SUPPORT_COLOR, SUPPORT_COLOR_TEMP as SUPPORT_COLOR_TEMP, SUPPORT_TRANSITION as SUPPORT_TRANSITION, SUPPORT_WHITE_VALUE as SUPPORT_WHITE_VALUE
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect
from typing import Any, Callable
from zwave_js_server.client import Client as ZwaveClient
from zwave_js_server.const import ColorComponent

LOGGER: Any
MULTI_COLOR_MAP: Any

async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: Callable) -> None: ...
def byte_to_zwave_brightness(value: int) -> int: ...

class ZwaveLight(ZWaveBaseEntity, LightEntity):
    _supports_color: bool = ...
    _supports_white_value: bool = ...
    _supports_color_temp: bool = ...
    _hs_color: Any = ...
    _white_value: Any = ...
    _color_temp: Any = ...
    _min_mireds: int = ...
    _max_mireds: int = ...
    _supported_features: Any = ...
    _target_value: Any = ...
    _dimming_duration: Any = ...
    def __init__(self, config_entry: ConfigEntry, client: ZwaveClient, info: ZwaveDiscoveryInfo) -> None: ...
    def on_value_update(self) -> None: ...
    @property
    def brightness(self) -> int: ...
    @property
    def is_on(self) -> bool: ...
    @property
    def hs_color(self) -> Union[tuple[float, float], None]: ...
    @property
    def white_value(self) -> Union[int, None]: ...
    @property
    def color_temp(self) -> Union[int, None]: ...
    @property
    def min_mireds(self) -> int: ...
    @property
    def max_mireds(self) -> int: ...
    @property
    def supported_features(self) -> int: ...
    async def async_turn_on(self, **kwargs: Any) -> None: ...
    async def async_turn_off(self, **kwargs: Any) -> None: ...
    async def _async_set_colors(self, colors: dict[ColorComponent, int]) -> None: ...
    async def _async_set_color(self, color: ColorComponent, new_value: int) -> None: ...
    async def _async_set_brightness(self, brightness: Union[int, None], transition: Union[int, None]=...) -> None: ...
    async def _async_set_transition_duration(self, duration: Union[int, None]=...) -> None: ...
    def _calculate_color_values(self) -> None: ...



from .connection import ActiveConnection as ActiveConnection
from .error import Disconnect as Disconnect
from homeassistant.auth.models import RefreshToken as RefreshToken, User as User
from homeassistant.components.http.ban import process_success_login as process_success_login, process_wrong_login as process_wrong_login
from homeassistant.const import __version__ as __version__
from typing import Any

TYPE_AUTH: str
TYPE_AUTH_INVALID: str
TYPE_AUTH_OK: str
TYPE_AUTH_REQUIRED: str
AUTH_MESSAGE_SCHEMA: Any

def auth_ok_message(): ...
def auth_required_message(): ...
def auth_invalid_message(message: Any): ...

class AuthPhase:
    _hass: Any = ...
    _send_message: Any = ...
    _logger: Any = ...
    _request: Any = ...
    _authenticated: bool = ...
    _connection: Any = ...
    def __init__(self, logger: Any, hass: Any, send_message: Any, request: Any) -> None: ...
    async def async_handle(self, msg: Any): ...
    async def _async_finish_auth(self, user: User, refresh_token: RefreshToken) -> ActiveConnection: ...



from . import GroupEntity as GroupEntity
from homeassistant.components import light as light
from homeassistant.components.light import ATTR_BRIGHTNESS as ATTR_BRIGHTNESS, ATTR_COLOR_TEMP as ATTR_COLOR_TEMP, ATTR_EFFECT as ATTR_EFFECT, ATTR_EFFECT_LIST as ATTR_EFFECT_LIST, ATTR_FLASH as ATTR_FLASH, ATTR_HS_COLOR as ATTR_HS_COLOR, ATTR_MAX_MIREDS as ATTR_MAX_MIREDS, ATTR_MIN_MIREDS as ATTR_MIN_MIREDS, ATTR_TRANSITION as ATTR_TRANSITION, ATTR_WHITE_VALUE as ATTR_WHITE_VALUE, PLATFORM_SCHEMA as PLATFORM_SCHEMA, SUPPORT_BRIGHTNESS as SUPPORT_BRIGHTNESS, SUPPORT_COLOR as SUPPORT_COLOR, SUPPORT_COLOR_TEMP as SUPPORT_COLOR_TEMP, SUPPORT_EFFECT as SUPPORT_EFFECT, SUPPORT_FLASH as SUPPORT_FLASH, SUPPORT_TRANSITION as SUPPORT_TRANSITION, SUPPORT_WHITE_VALUE as SUPPORT_WHITE_VALUE
from homeassistant.const import ATTR_ENTITY_ID as ATTR_ENTITY_ID, ATTR_SUPPORTED_FEATURES as ATTR_SUPPORTED_FEATURES, CONF_ENTITIES as CONF_ENTITIES, CONF_NAME as CONF_NAME, STATE_ON as STATE_ON, STATE_UNAVAILABLE as STATE_UNAVAILABLE
from homeassistant.core import CoreState as CoreState, HomeAssistant as HomeAssistant, State as State
from homeassistant.helpers.event import async_track_state_change_event as async_track_state_change_event
from homeassistant.helpers.typing import ConfigType as ConfigType
from typing import Any, Callable, Iterator

DEFAULT_NAME: str
SUPPORT_GROUP_LIGHT: Any

async def async_setup_platform(hass: HomeAssistant, config: ConfigType, async_add_entities: Any, discovery_info: Any=...) -> None: ...

class LightGroup(GroupEntity, light.LightEntity):
    _name: Any = ...
    _entity_ids: Any = ...
    _is_on: bool = ...
    _available: bool = ...
    _icon: str = ...
    _brightness: Any = ...
    _hs_color: Any = ...
    _color_temp: Any = ...
    _min_mireds: int = ...
    _max_mireds: int = ...
    _white_value: Any = ...
    _effect_list: Any = ...
    _effect: Any = ...
    _supported_features: int = ...
    def __init__(self, name: str, entity_ids: list[str]) -> None: ...
    async def async_added_to_hass(self) -> None: ...
    @property
    def name(self) -> str: ...
    @property
    def is_on(self) -> bool: ...
    @property
    def available(self) -> bool: ...
    @property
    def icon(self): ...
    @property
    def brightness(self) -> Union[int, None]: ...
    @property
    def hs_color(self) -> Union[tuple[float, float], None]: ...
    @property
    def color_temp(self) -> Union[int, None]: ...
    @property
    def min_mireds(self) -> int: ...
    @property
    def max_mireds(self) -> int: ...
    @property
    def white_value(self) -> Union[int, None]: ...
    @property
    def effect_list(self) -> Union[list[str], None]: ...
    @property
    def effect(self) -> Union[str, None]: ...
    @property
    def supported_features(self) -> int: ...
    @property
    def should_poll(self) -> bool: ...
    @property
    def extra_state_attributes(self): ...
    async def async_turn_on(self, **kwargs: Any) -> None: ...
    async def async_turn_off(self, **kwargs: Any) -> None: ...
    async def async_update(self) -> None: ...

def _find_state_attributes(states: list[State], key: str) -> Iterator[Any]: ...
def _mean_int(*args: Any): ...
def _mean_tuple(*args: Any): ...
def _reduce_attribute(states: list[State], key: str, default: Union[Any, None]=..., reduce: Callable[..., Any]=...) -> Any: ...



from .const import DATA_CONNECTIONS as DATA_CONNECTIONS, SIGNAL_WEBSOCKET_CONNECTED as SIGNAL_WEBSOCKET_CONNECTED, SIGNAL_WEBSOCKET_DISCONNECTED as SIGNAL_WEBSOCKET_DISCONNECTED
from homeassistant.components.sensor import SensorEntity as SensorEntity
from homeassistant.core import callback as callback
from typing import Any, Optional

async def async_setup_platform(hass: Any, config: Any, async_add_entities: Any, discovery_info: Optional[Any] = ...) -> None: ...

class APICount(SensorEntity):
    count: int = ...
    def __init__(self) -> None: ...
    async def async_added_to_hass(self) -> None: ...
    @property
    def name(self): ...
    @property
    def state(self): ...
    @property
    def unit_of_measurement(self): ...
    def _update_count(self) -> None: ...



from . import permissions as perm_mdl
from .const import GROUP_ID_ADMIN as GROUP_ID_ADMIN
from datetime import datetime, timedelta
from homeassistant.const import __version__ as __version__
from typing import Any, NamedTuple

TOKEN_TYPE_NORMAL: str
TOKEN_TYPE_SYSTEM: str
TOKEN_TYPE_LONG_LIVED_ACCESS_TOKEN: str

class Group:
    name: Union[str, None] = ...
    policy: perm_mdl.PolicyType = ...
    id: str = ...
    system_generated: bool = ...
    def __init__(self, name: Any, policy: Any, id: Any, system_generated: Any) -> None: ...
    def __lt__(self, other: Any) -> Any: ...
    def __le__(self, other: Any) -> Any: ...
    def __gt__(self, other: Any) -> Any: ...
    def __ge__(self, other: Any) -> Any: ...

class User:
    name: Union[str, None] = ...
    perm_lookup: perm_mdl.PermissionLookup = ...
    id: str = ...
    is_owner: bool = ...
    is_active: bool = ...
    system_generated: bool = ...
    groups: list[Group] = ...
    credentials: list[Credentials] = ...
    refresh_tokens: dict[str, RefreshToken] = ...
    _permissions: Union[perm_mdl.PolicyPermissions, None] = ...
    @property
    def permissions(self) -> perm_mdl.AbstractPermissions: ...
    @property
    def is_admin(self) -> bool: ...
    def invalidate_permission_cache(self) -> None: ...
    def __init__(self, name: Any, perm_lookup: Any, id: Any, is_owner: Any, is_active: Any, system_generated: Any, groups: Any, credentials: Any, refresh_tokens: Any, permissions: Any) -> None: ...
    def __lt__(self, other: Any) -> Any: ...
    def __le__(self, other: Any) -> Any: ...
    def __gt__(self, other: Any) -> Any: ...
    def __ge__(self, other: Any) -> Any: ...

class RefreshToken:
    user: User = ...
    client_id: Union[str, None] = ...
    access_token_expiration: timedelta = ...
    client_name: Union[str, None] = ...
    client_icon: Union[str, None] = ...
    token_type: str = ...
    id: str = ...
    created_at: datetime = ...
    token: str = ...
    jwt_key: str = ...
    last_used_at: Union[datetime, None] = ...
    last_used_ip: Union[str, None] = ...
    credential: Union[Credentials, None] = ...
    version: Union[str, None] = ...
    def __init__(self, user: Any, client_id: Any, access_token_expiration: Any, client_name: Any, client_icon: Any, token_type: Any, id: Any, created_at: Any, token: Any, jwt_key: Any, last_used_at: Any, last_used_ip: Any, credential: Any, version: Any) -> None: ...
    def __lt__(self, other: Any) -> Any: ...
    def __le__(self, other: Any) -> Any: ...
    def __gt__(self, other: Any) -> Any: ...
    def __ge__(self, other: Any) -> Any: ...

class Credentials:
    auth_provider_type: str = ...
    auth_provider_id: Union[str, None] = ...
    data: dict = ...
    id: str = ...
    is_new: bool = ...
    def __init__(self, auth_provider_type: Any, auth_provider_id: Any, data: Any, id: Any, is_new: Any) -> None: ...
    def __lt__(self, other: Any) -> Any: ...
    def __le__(self, other: Any) -> Any: ...
    def __gt__(self, other: Any) -> Any: ...
    def __ge__(self, other: Any) -> Any: ...

class UserMeta(NamedTuple):
    name: Union[str, None]
    is_active: bool



from .const import BPUP_SUBS as BPUP_SUBS, DOMAIN as DOMAIN, HUB as HUB
from .entity import BondEntity as BondEntity
from .utils import BondDevice as BondDevice, BondHub as BondHub
from bond_api import BPUPSubscriptions as BPUPSubscriptions
from homeassistant.components.switch import SwitchEntity as SwitchEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity import Entity as Entity
from typing import Any, Callable

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: Callable[[list[Entity], bool], None]) -> None: ...

class BondSwitch(BondEntity, SwitchEntity):
    _power: Any = ...
    def __init__(self, hub: BondHub, device: BondDevice, bpup_subs: BPUPSubscriptions) -> None: ...
    def _apply_state(self, state: dict) -> None: ...
    @property
    def is_on(self) -> bool: ...
    async def async_turn_on(self, **kwargs: Any) -> None: ...
    async def async_turn_off(self, **kwargs: Any) -> None: ...



import jinja2
from contextvars import ContextVar
from homeassistant.const import ATTR_ENTITY_ID as ATTR_ENTITY_ID, ATTR_LATITUDE as ATTR_LATITUDE, ATTR_LONGITUDE as ATTR_LONGITUDE, ATTR_UNIT_OF_MEASUREMENT as ATTR_UNIT_OF_MEASUREMENT, LENGTH_METERS as LENGTH_METERS, STATE_UNKNOWN as STATE_UNKNOWN
from homeassistant.core import HomeAssistant as HomeAssistant, State as State, callback as callback, split_entity_id as split_entity_id, valid_entity_id as valid_entity_id
from homeassistant.exceptions import TemplateError as TemplateError
from homeassistant.helpers import entity_registry as entity_registry
from homeassistant.helpers.typing import TemplateVarsType as TemplateVarsType
from homeassistant.loader import bind_hass as bind_hass
from homeassistant.util import convert as convert
from homeassistant.util.async_ import run_callback_threadsafe as run_callback_threadsafe
from homeassistant.util.thread import ThreadWithException as ThreadWithException
from jinja2.sandbox import ImmutableSandboxedEnvironment
from typing import Any, Generator, Iterable, Optional

_LOGGER: Any
_SENTINEL: Any
DATE_STR_FORMAT: str
_RENDER_INFO: str
_ENVIRONMENT: str
_ENVIRONMENT_LIMITED: str
_RE_JINJA_DELIMITERS: Any
_IS_NUMERIC: Any
_RESERVED_NAMES: Any
_GROUP_DOMAIN_PREFIX: str
_COLLECTABLE_STATE_ATTRIBUTES: Any
ALL_STATES_RATE_LIMIT: Any
DOMAIN_STATES_RATE_LIMIT: Any
template_cv: ContextVar[Union[str, None]]

def attach(hass: HomeAssistant, obj: Any) -> None: ...
def render_complex(value: Any, variables: TemplateVarsType=..., limited: bool=...) -> Any: ...
def is_complex(value: Any) -> bool: ...
def is_template_string(maybe_template: str) -> bool: ...

class ResultWrapper:
    render_result: Union[str, None]

def gen_result_wrapper(kls: Any): ...

class TupleWrapper(tuple, ResultWrapper):
    def __new__(cls: Any, value: tuple, *, render_result: Union[str, None]=...) -> TupleWrapper: ...
    render_result: Any = ...
    def __init__(self, value: tuple, *, render_result: Union[str, None]=...) -> None: ...
    def __str__(self) -> str: ...

RESULT_WRAPPERS: dict[type, type]

def _true(arg: Any) -> bool: ...
def _false(arg: Any) -> bool: ...

class RenderInfo:
    template: Any = ...
    filter_lifecycle: Any = ...
    filter: Any = ...
    _result: Any = ...
    is_static: bool = ...
    exception: Any = ...
    all_states: bool = ...
    all_states_lifecycle: bool = ...
    domains: Any = ...
    domains_lifecycle: Any = ...
    entities: Any = ...
    rate_limit: Any = ...
    has_time: bool = ...
    def __init__(self, template: Any) -> None: ...
    def __repr__(self) -> str: ...
    def _filter_domains_and_entities(self, entity_id: str) -> bool: ...
    def _filter_entities(self, entity_id: str) -> bool: ...
    def _filter_lifecycle_domains(self, entity_id: str) -> bool: ...
    def result(self) -> str: ...
    def _freeze_static(self) -> None: ...
    def _freeze_sets(self) -> None: ...
    def _freeze(self) -> None: ...

class Template:
    __slots__: Any = ...
    template: Any = ...
    _compiled_code: Any = ...
    _compiled: Any = ...
    hass: Any = ...
    is_static: Any = ...
    _limited: Any = ...
    def __init__(self, template: Any, hass: Optional[Any] = ...) -> None: ...
    @property
    def _env(self) -> TemplateEnvironment: ...
    def ensure_valid(self) -> None: ...
    def render(self, variables: TemplateVarsType=..., parse_result: bool=..., limited: bool=..., **kwargs: Any) -> Any: ...
    def async_render(self, variables: TemplateVarsType=..., parse_result: bool=..., limited: bool=..., **kwargs: Any) -> Any: ...
    def _parse_result(self, render_result: str) -> Any: ...
    async def async_render_will_timeout(self, timeout: float, variables: TemplateVarsType=..., **kwargs: Any) -> bool: ...
    def async_render_to_info(self, variables: TemplateVarsType=..., **kwargs: Any) -> RenderInfo: ...
    def render_with_possible_json_value(self, value: Any, error_value: Any = ...): ...
    def async_render_with_possible_json_value(self, value: Any, error_value: Any = ..., variables: Optional[Any] = ...): ...
    def _ensure_compiled(self, limited: bool=...) -> jinja2.Template: ...
    def __eq__(self, other: Any) -> Any: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...

class AllStates:
    _hass: Any = ...
    def __init__(self, hass: HomeAssistant) -> None: ...
    def __getattr__(self, name: Any): ...
    __getitem__: Any = ...
    def _collect_all(self) -> None: ...
    def _collect_all_lifecycle(self) -> None: ...
    def __iter__(self) -> Any: ...
    def __len__(self) -> int: ...
    def __call__(self, entity_id: Any): ...
    def __repr__(self) -> str: ...

class DomainStates:
    _hass: Any = ...
    _domain: Any = ...
    def __init__(self, hass: HomeAssistant, domain: str) -> None: ...
    def __getattr__(self, name: Any): ...
    __getitem__: Any = ...
    def _collect_domain(self) -> None: ...
    def _collect_domain_lifecycle(self) -> None: ...
    def __iter__(self) -> Any: ...
    def __len__(self) -> int: ...
    def __repr__(self) -> str: ...

class TemplateState(State):
    __slots__: Any = ...
    _hass: Any = ...
    _state: Any = ...
    _collect: Any = ...
    def __init__(self, hass: HomeAssistant, state: State, collect: bool=...) -> None: ...
    def _collect_state(self) -> None: ...
    def __getitem__(self, item: Any): ...
    @property
    def entity_id(self): ...
    @property
    def state(self): ...
    @property
    def attributes(self): ...
    @property
    def last_changed(self): ...
    @property
    def last_updated(self): ...
    @property
    def context(self): ...
    @property
    def domain(self): ...
    @property
    def object_id(self): ...
    @property
    def name(self): ...
    @property
    def state_with_unit(self) -> str: ...
    def __eq__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...

def _collect_state(hass: HomeAssistant, entity_id: str) -> None: ...
def _state_generator(hass: HomeAssistant, domain: Union[str, None]) -> Generator: ...
def _get_state_if_valid(hass: HomeAssistant, entity_id: str) -> Union[TemplateState, None]: ...
def _get_state(hass: HomeAssistant, entity_id: str) -> Union[TemplateState, None]: ...
def _get_template_state_from_state(hass: HomeAssistant, entity_id: str, state: Union[State, None]) -> Union[TemplateState, None]: ...
def _resolve_state(hass: HomeAssistant, entity_id_or_state: Any) -> Union[State, TemplateState, None]: ...
def result_as_boolean(template_result: Union[str, None]) -> bool: ...
def expand(hass: HomeAssistant, *args: Any) -> Iterable[State]: ...
def device_entities(hass: HomeAssistant, device_id: str) -> Iterable[str]: ...
def closest(hass: Any, *args: Any): ...
def closest_filter(hass: Any, *args: Any): ...
def distance(hass: Any, *args: Any): ...
def is_state(hass: HomeAssistant, entity_id: str, state: State) -> bool: ...
def is_state_attr(hass: Any, entity_id: Any, name: Any, value: Any): ...
def state_attr(hass: Any, entity_id: Any, name: Any): ...
def now(hass: Any): ...
def utcnow(hass: Any): ...
def forgiving_round(value: Any, precision: int = ..., method: str = ...): ...
def multiply(value: Any, amount: Any): ...
def logarithm(value: Any, base: Any = ...): ...
def sine(value: Any): ...
def cosine(value: Any): ...
def tangent(value: Any): ...
def arc_sine(value: Any): ...
def arc_cosine(value: Any): ...
def arc_tangent(value: Any): ...
def arc_tangent2(*args: Any): ...
def square_root(value: Any): ...
def timestamp_custom(value: Any, date_format: Any = ..., local: bool = ...): ...
def timestamp_local(value: Any): ...
def timestamp_utc(value: Any): ...
def forgiving_as_timestamp(value: Any): ...
def strptime(string: Any, fmt: Any): ...
def fail_when_undefined(value: Any): ...
def forgiving_float(value: Any): ...
def regex_match(value: Any, find: str = ..., ignorecase: bool = ...): ...
def regex_replace(value: str = ..., find: str = ..., replace: str = ..., ignorecase: bool = ...): ...
def regex_search(value: Any, find: str = ..., ignorecase: bool = ...): ...
def regex_findall_index(value: Any, find: str = ..., index: int = ..., ignorecase: bool = ...): ...
def bitwise_and(first_value: Any, second_value: Any): ...
def bitwise_or(first_value: Any, second_value: Any): ...
def base64_encode(value: Any): ...
def base64_decode(value: Any): ...
def ordinal(value: Any): ...
def from_json(value: Any): ...
def to_json(value: Any): ...
def random_every_time(context: Any, values: Any): ...
def relative_time(value: Any): ...
def urlencode(value: Any): ...
def _render_with_context(template_str: str, template: jinja2.Template, **kwargs: Any) -> str: ...

class LoggingUndefined(jinja2.Undefined):
    def _log_message(self) -> None: ...
    def _fail_with_undefined_error(self, *args: Any, **kwargs: Any): ...
    def __str__(self): ...
    def __iter__(self) -> Any: ...
    def __bool__(self): ...

class TemplateEnvironment(ImmutableSandboxedEnvironment):
    hass: Any = ...
    template_cache: Any = ...
    def __init__(self, hass: Any, limited: bool = ...): ...
    def is_safe_callable(self, obj: Any): ...
    def is_safe_attribute(self, obj: Any, attr: Any, value: Any): ...
    def compile(self, source: Any, name: Optional[Any] = ..., filename: Optional[Any] = ..., raw: bool = ..., defer_init: bool = ...): ...

_NO_HASS_ENV: Any



from .const import KEY_AUTHENTICATED as KEY_AUTHENTICATED, KEY_HASS as KEY_HASS
from aiohttp import web
from aiohttp.typedefs import LooseHeaders as LooseHeaders
from homeassistant import exceptions as exceptions
from homeassistant.const import CONTENT_TYPE_JSON as CONTENT_TYPE_JSON, HTTP_OK as HTTP_OK, HTTP_SERVICE_UNAVAILABLE as HTTP_SERVICE_UNAVAILABLE
from homeassistant.core import Context as Context, is_callback as is_callback
from homeassistant.helpers.json import JSONEncoder as JSONEncoder
from typing import Any, Callable

_LOGGER: Any

class HomeAssistantView:
    url: Union[str, None] = ...
    extra_urls: list[str] = ...
    requires_auth: bool = ...
    cors_allowed: bool = ...
    @staticmethod
    def context(request: web.Request) -> Context: ...
    @staticmethod
    def json(result: Any, status_code: int=..., headers: Union[LooseHeaders, None]=...) -> web.Response: ...
    def json_message(self, message: str, status_code: int=..., message_code: Union[str, None]=..., headers: Union[LooseHeaders, None]=...) -> web.Response: ...
    def register(self, app: web.Application, router: web.UrlDispatcher) -> None: ...

def request_handler_factory(view: HomeAssistantView, handler: Callable) -> Callable: ...



from .const import DOMAIN as DOMAIN
from .knx_entity import KnxEntity as KnxEntity
from homeassistant.components.sensor import DEVICE_CLASSES as DEVICE_CLASSES, SensorEntity as SensorEntity
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.typing import ConfigType as ConfigType, DiscoveryInfoType as DiscoveryInfoType, StateType as StateType
from typing import Any, Callable, Iterable
from xknx.devices import Sensor as XknxSensor

async def async_setup_platform(hass: HomeAssistant, config: ConfigType, async_add_entities: Callable[[Iterable[Entity]], None], discovery_info: Union[DiscoveryInfoType, None]=...) -> None: ...

class KNXSensor(KnxEntity, SensorEntity):
    _device: Any
    def __init__(self, device: XknxSensor) -> None: ...
    @property
    def state(self) -> StateType: ...
    @property
    def unit_of_measurement(self) -> Union[str, None]: ...
    @property
    def device_class(self) -> Union[str, None]: ...
    @property
    def force_update(self) -> bool: ...



from homeassistant.const import SERVICE_TOGGLE as SERVICE_TOGGLE, SERVICE_TURN_OFF as SERVICE_TURN_OFF, SERVICE_TURN_ON as SERVICE_TURN_ON, STATE_ON as STATE_ON
from homeassistant.helpers.config_validation import PLATFORM_SCHEMA as PLATFORM_SCHEMA, PLATFORM_SCHEMA_BASE as PLATFORM_SCHEMA_BASE
from homeassistant.helpers.entity import ToggleEntity as ToggleEntity
from homeassistant.helpers.entity_component import EntityComponent as EntityComponent
from homeassistant.loader import bind_hass as bind_hass
from typing import Any

DOMAIN: str
SCAN_INTERVAL: Any
ENTITY_ID_FORMAT: Any
ATTR_TODAY_ENERGY_KWH: str
ATTR_CURRENT_POWER_W: str
MIN_TIME_BETWEEN_SCANS: Any
PROP_TO_ATTR: Any
DEVICE_CLASS_OUTLET: str
DEVICE_CLASS_SWITCH: str
DEVICE_CLASSES: Any
DEVICE_CLASSES_SCHEMA: Any
_LOGGER: Any

def is_on(hass: Any, entity_id: Any): ...
async def async_setup(hass: Any, config: Any): ...
async def async_setup_entry(hass: Any, entry: Any): ...
async def async_unload_entry(hass: Any, entry: Any): ...

class SwitchEntity(ToggleEntity):
    @property
    def current_power_w(self) -> None: ...
    @property
    def today_energy_kwh(self) -> None: ...
    @property
    def is_standby(self) -> None: ...
    @property
    def state_attributes(self): ...
    @property
    def device_class(self) -> None: ...

class SwitchDevice(SwitchEntity):
    def __init_subclass__(cls, **kwargs: Any) -> None: ...



from . import HuaweiLteBaseEntity as HuaweiLteBaseEntity
from .const import DOMAIN as DOMAIN, KEY_DIALUP_MOBILE_DATASWITCH as KEY_DIALUP_MOBILE_DATASWITCH
from homeassistant.components.switch import DEVICE_CLASS_SWITCH as DEVICE_CLASS_SWITCH, SwitchEntity as SwitchEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_URL as CONF_URL
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.typing import HomeAssistantType as HomeAssistantType
from typing import Any, Callable

_LOGGER: Any

async def async_setup_entry(hass: HomeAssistantType, config_entry: ConfigEntry, async_add_entities: Callable[[list[Entity], bool], None]) -> None: ...

class HuaweiLteBaseSwitch(HuaweiLteBaseEntity, SwitchEntity):
    key: str
    item: str
    _raw_state: Union[str, None] = ...
    def _turn(self, state: bool) -> None: ...
    def turn_on(self, **kwargs: Any) -> None: ...
    def turn_off(self, **kwargs: Any) -> None: ...
    @property
    def device_class(self) -> str: ...
    async def async_added_to_hass(self) -> None: ...
    async def async_will_remove_from_hass(self) -> None: ...
    _available: bool = ...
    async def async_update(self) -> None: ...
    def __init__(self, router: Any, available: Any, unsub_handlers: Any, raw_state: Any) -> None: ...
    def __lt__(self, other: Any) -> Any: ...
    def __le__(self, other: Any) -> Any: ...
    def __gt__(self, other: Any) -> Any: ...
    def __ge__(self, other: Any) -> Any: ...

class HuaweiLteMobileDataSwitch(HuaweiLteBaseSwitch):
    key: Any = ...
    item: str = ...
    def __attrs_post_init__(self) -> None: ...
    @property
    def _entity_name(self) -> str: ...
    @property
    def _device_unique_id(self) -> str: ...
    @property
    def is_on(self) -> bool: ...
    _raw_state: Any = ...
    def _turn(self, state: bool) -> None: ...
    @property
    def icon(self) -> str: ...
    def __init__(self, router: Any, available: Any, unsub_handlers: Any, raw_state: Any) -> None: ...
    def __lt__(self, other: Any) -> Any: ...
    def __le__(self, other: Any) -> Any: ...
    def __gt__(self, other: Any) -> Any: ...
    def __ge__(self, other: Any) -> Any: ...



from homeassistant.const import HTTP_OK as HTTP_OK
from multidict import MultiDict
from typing import Any

class MockStreamReader:
    _content: Any = ...
    def __init__(self, content: bytes) -> None: ...
    async def read(self, byte_count: int=...) -> bytes: ...

class MockRequest:
    mock_source: Union[str, None] = ...
    method: Any = ...
    url: Any = ...
    status: Any = ...
    headers: Any = ...
    query_string: Any = ...
    _content: Any = ...
    def __init__(self, content: bytes, mock_source: str, method: str=..., status: int=..., headers: Union[dict[str, str], None]=..., query_string: Union[str, None]=..., url: str=...) -> None: ...
    @property
    def query(self) -> MultiDict[str]: ...
    @property
    def _text(self) -> str: ...
    @property
    def content(self) -> MockStreamReader: ...
    async def json(self) -> Any: ...
    async def post(self) -> MultiDict[str]: ...
    async def text(self) -> str: ...



import yaml
from typing import Any

class NodeListClass(list): ...
class NodeStrClass(str): ...

class Input:
    name: str
    @classmethod
    def from_node(cls: Any, loader: yaml.Loader, node: yaml.nodes.Node) -> Input: ...
    def __init__(self, name: Any) -> None: ...



from homeassistant.const import PRECISION_TENTHS as PRECISION_TENTHS, PRECISION_WHOLE as PRECISION_WHOLE, TEMP_CELSIUS as TEMP_CELSIUS
from homeassistant.helpers.config_validation import PLATFORM_SCHEMA as PLATFORM_SCHEMA, PLATFORM_SCHEMA_BASE as PLATFORM_SCHEMA_BASE
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.entity_component import EntityComponent as EntityComponent
from typing import Any

_LOGGER: Any
ATTR_CONDITION_CLASS: str
ATTR_CONDITION_CLEAR_NIGHT: str
ATTR_CONDITION_CLOUDY: str
ATTR_CONDITION_EXCEPTIONAL: str
ATTR_CONDITION_FOG: str
ATTR_CONDITION_HAIL: str
ATTR_CONDITION_LIGHTNING: str
ATTR_CONDITION_LIGHTNING_RAINY: str
ATTR_CONDITION_PARTLYCLOUDY: str
ATTR_CONDITION_POURING: str
ATTR_CONDITION_RAINY: str
ATTR_CONDITION_SNOWY: str
ATTR_CONDITION_SNOWY_RAINY: str
ATTR_CONDITION_SUNNY: str
ATTR_CONDITION_WINDY: str
ATTR_CONDITION_WINDY_VARIANT: str
ATTR_FORECAST: str
ATTR_FORECAST_CONDITION: str
ATTR_FORECAST_PRECIPITATION: str
ATTR_FORECAST_PRECIPITATION_PROBABILITY: str
ATTR_FORECAST_PRESSURE: str
ATTR_FORECAST_TEMP: str
ATTR_FORECAST_TEMP_LOW: str
ATTR_FORECAST_TIME: str
ATTR_FORECAST_WIND_BEARING: str
ATTR_FORECAST_WIND_SPEED: str
ATTR_WEATHER_ATTRIBUTION: str
ATTR_WEATHER_HUMIDITY: str
ATTR_WEATHER_OZONE: str
ATTR_WEATHER_PRESSURE: str
ATTR_WEATHER_TEMPERATURE: str
ATTR_WEATHER_VISIBILITY: str
ATTR_WEATHER_WIND_BEARING: str
ATTR_WEATHER_WIND_SPEED: str
DOMAIN: str
ENTITY_ID_FORMAT: Any
SCAN_INTERVAL: Any

async def async_setup(hass: Any, config: Any): ...
async def async_setup_entry(hass: Any, entry: Any): ...
async def async_unload_entry(hass: Any, entry: Any): ...

class WeatherEntity(Entity):
    @property
    def temperature(self) -> None: ...
    @property
    def temperature_unit(self) -> None: ...
    @property
    def pressure(self) -> None: ...
    @property
    def humidity(self) -> None: ...
    @property
    def wind_speed(self) -> None: ...
    @property
    def wind_bearing(self) -> None: ...
    @property
    def ozone(self) -> None: ...
    @property
    def attribution(self) -> None: ...
    @property
    def visibility(self) -> None: ...
    @property
    def forecast(self) -> None: ...
    @property
    def precision(self): ...
    @property
    def state_attributes(self): ...
    @property
    def state(self): ...
    @property
    def condition(self) -> None: ...



from homeassistant.const import SERVICE_CLOSE_COVER as SERVICE_CLOSE_COVER, SERVICE_CLOSE_COVER_TILT as SERVICE_CLOSE_COVER_TILT, SERVICE_OPEN_COVER as SERVICE_OPEN_COVER, SERVICE_OPEN_COVER_TILT as SERVICE_OPEN_COVER_TILT, SERVICE_SET_COVER_POSITION as SERVICE_SET_COVER_POSITION, SERVICE_SET_COVER_TILT_POSITION as SERVICE_SET_COVER_TILT_POSITION, SERVICE_STOP_COVER as SERVICE_STOP_COVER, SERVICE_STOP_COVER_TILT as SERVICE_STOP_COVER_TILT, SERVICE_TOGGLE as SERVICE_TOGGLE, SERVICE_TOGGLE_COVER_TILT as SERVICE_TOGGLE_COVER_TILT, STATE_CLOSED as STATE_CLOSED, STATE_CLOSING as STATE_CLOSING, STATE_OPEN as STATE_OPEN, STATE_OPENING as STATE_OPENING
from homeassistant.helpers.config_validation import PLATFORM_SCHEMA as PLATFORM_SCHEMA, PLATFORM_SCHEMA_BASE as PLATFORM_SCHEMA_BASE
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.entity_component import EntityComponent as EntityComponent
from homeassistant.loader import bind_hass as bind_hass
from typing import Any

_LOGGER: Any
DOMAIN: str
SCAN_INTERVAL: Any
ENTITY_ID_FORMAT: Any
DEVICE_CLASS_AWNING: str
DEVICE_CLASS_BLIND: str
DEVICE_CLASS_CURTAIN: str
DEVICE_CLASS_DAMPER: str
DEVICE_CLASS_DOOR: str
DEVICE_CLASS_GARAGE: str
DEVICE_CLASS_GATE: str
DEVICE_CLASS_SHADE: str
DEVICE_CLASS_SHUTTER: str
DEVICE_CLASS_WINDOW: str
DEVICE_CLASSES: Any
DEVICE_CLASSES_SCHEMA: Any
SUPPORT_OPEN: int
SUPPORT_CLOSE: int
SUPPORT_SET_POSITION: int
SUPPORT_STOP: int
SUPPORT_OPEN_TILT: int
SUPPORT_CLOSE_TILT: int
SUPPORT_STOP_TILT: int
SUPPORT_SET_TILT_POSITION: int
ATTR_CURRENT_POSITION: str
ATTR_CURRENT_TILT_POSITION: str
ATTR_POSITION: str
ATTR_TILT_POSITION: str

def is_closed(hass: Any, entity_id: Any): ...
async def async_setup(hass: Any, config: Any): ...
async def async_setup_entry(hass: Any, entry: Any): ...
async def async_unload_entry(hass: Any, entry: Any): ...

class CoverEntity(Entity):
    @property
    def current_cover_position(self) -> None: ...
    @property
    def current_cover_tilt_position(self) -> None: ...
    @property
    def state(self): ...
    @property
    def state_attributes(self): ...
    @property
    def supported_features(self): ...
    @property
    def is_opening(self) -> None: ...
    @property
    def is_closing(self) -> None: ...
    @property
    def is_closed(self) -> None: ...
    def open_cover(self, **kwargs: Any) -> None: ...
    async def async_open_cover(self, **kwargs: Any) -> None: ...
    def close_cover(self, **kwargs: Any) -> None: ...
    async def async_close_cover(self, **kwargs: Any) -> None: ...
    def toggle(self, **kwargs: Any) -> None: ...
    async def async_toggle(self, **kwargs: Any) -> None: ...
    def set_cover_position(self, **kwargs: Any) -> None: ...
    async def async_set_cover_position(self, **kwargs: Any) -> None: ...
    def stop_cover(self, **kwargs: Any) -> None: ...
    async def async_stop_cover(self, **kwargs: Any) -> None: ...
    def open_cover_tilt(self, **kwargs: Any) -> None: ...
    async def async_open_cover_tilt(self, **kwargs: Any) -> None: ...
    def close_cover_tilt(self, **kwargs: Any) -> None: ...
    async def async_close_cover_tilt(self, **kwargs: Any) -> None: ...
    def set_cover_tilt_position(self, **kwargs: Any) -> None: ...
    async def async_set_cover_tilt_position(self, **kwargs: Any) -> None: ...
    def stop_cover_tilt(self, **kwargs: Any) -> None: ...
    async def async_stop_cover_tilt(self, **kwargs: Any) -> None: ...
    def toggle_tilt(self, **kwargs: Any) -> None: ...
    async def async_toggle_tilt(self, **kwargs: Any) -> None: ...

class CoverDevice(CoverEntity):
    def __init_subclass__(cls, **kwargs: Any) -> None: ...



from . import DOMAIN as DOMAIN
from homeassistant.const import ATTR_ENTITY_ID as ATTR_ENTITY_ID, SERVICE_LOCK as SERVICE_LOCK, SERVICE_UNLOCK as SERVICE_UNLOCK, STATE_LOCKED as STATE_LOCKED, STATE_UNLOCKED as STATE_UNLOCKED
from homeassistant.core import Context as Context, State as State
from homeassistant.helpers.typing import HomeAssistantType as HomeAssistantType
from typing import Any, Iterable

_LOGGER: Any
VALID_STATES: Any

async def _async_reproduce_state(hass: HomeAssistantType, state: State, *, context: Union[Context, None]=..., reproduce_options: Union[dict[str, Any], None]=...) -> None: ...
async def async_reproduce_states(hass: HomeAssistantType, states: Iterable[State], *, context: Union[Context, None]=..., reproduce_options: Union[dict[str, Any], None]=...) -> None: ...



import voluptuous as vol
from . import commands as commands, connection as connection, const as const, decorators as decorators, http as http, messages as messages
from .connection import ActiveConnection as ActiveConnection
from .const import ERR_HOME_ASSISTANT_ERROR as ERR_HOME_ASSISTANT_ERROR, ERR_INVALID_FORMAT as ERR_INVALID_FORMAT, ERR_NOT_FOUND as ERR_NOT_FOUND, ERR_NOT_SUPPORTED as ERR_NOT_SUPPORTED, ERR_TEMPLATE_ERROR as ERR_TEMPLATE_ERROR, ERR_TIMEOUT as ERR_TIMEOUT, ERR_UNAUTHORIZED as ERR_UNAUTHORIZED, ERR_UNKNOWN_COMMAND as ERR_UNKNOWN_COMMAND, ERR_UNKNOWN_ERROR as ERR_UNKNOWN_ERROR
from .decorators import async_response as async_response, require_admin as require_admin, websocket_command as websocket_command, ws_require_user as ws_require_user
from .messages import BASE_COMMAND_MESSAGE_SCHEMA as BASE_COMMAND_MESSAGE_SCHEMA, error_message as error_message, event_message as event_message, result_message as result_message
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.loader import bind_hass as bind_hass
from typing import Any

DOMAIN: Any
DEPENDENCIES: Any

def async_register_command(hass: HomeAssistant, command_or_handler: Union[str, const.WebSocketCommandHandler], handler: Union[const.WebSocketCommandHandler, None]=..., schema: Union[vol.Schema, None]=...) -> None: ...
async def async_setup(hass: Any, config: Any): ...



from . import ATTR_BRIGHTNESS_PCT as ATTR_BRIGHTNESS_PCT, ATTR_RGB_COLOR as ATTR_RGB_COLOR, DOMAIN as DOMAIN, SERVICE_TURN_ON as SERVICE_TURN_ON, SUPPORT_BRIGHTNESS as SUPPORT_BRIGHTNESS, SUPPORT_COLOR as SUPPORT_COLOR
from homeassistant.const import ATTR_ENTITY_ID as ATTR_ENTITY_ID
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers import intent as intent
from typing import Any

INTENT_SET: str

async def async_setup_intents(hass: HomeAssistant) -> None: ...

class SetIntentHandler(intent.IntentHandler):
    intent_type: Any = ...
    slot_schema: Any = ...
    async def async_handle(self, intent_obj: intent.Intent) -> intent.IntentResponse: ...



from .const import ATTR_COUNTER as ATTR_COUNTER, DOMAIN as DOMAIN
from .knx_entity import KnxEntity as KnxEntity
from homeassistant.components.binary_sensor import BinarySensorEntity as BinarySensorEntity, DEVICE_CLASSES as DEVICE_CLASSES
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.typing import ConfigType as ConfigType, DiscoveryInfoType as DiscoveryInfoType
from typing import Any, Callable, Iterable
from xknx.devices import BinarySensor as XknxBinarySensor

async def async_setup_platform(hass: HomeAssistant, config: ConfigType, async_add_entities: Callable[[Iterable[Entity]], None], discovery_info: Union[DiscoveryInfoType, None]=...) -> None: ...

class KNXBinarySensor(KnxEntity, BinarySensorEntity):
    _device: Any
    def __init__(self, device: XknxBinarySensor) -> None: ...
    @property
    def device_class(self) -> Union[str, None]: ...
    @property
    def is_on(self) -> bool: ...
    @property
    def extra_state_attributes(self) -> Union[dict[str, Any], None]: ...
    @property
    def force_update(self) -> bool: ...



from typing import Any

MAJOR_VERSION: int
MINOR_VERSION: int
PATCH_VERSION: str
__short_version__: Any
__version__: Any
REQUIRED_PYTHON_VER: Any
REQUIRED_NEXT_PYTHON_VER: Any
REQUIRED_NEXT_PYTHON_DATE: str
PLATFORM_FORMAT: str
MATCH_ALL: str
ENTITY_MATCH_NONE: str
ENTITY_MATCH_ALL: str
DEVICE_DEFAULT_NAME: str
SUN_EVENT_SUNSET: str
SUN_EVENT_SUNRISE: str
CONF_ABOVE: str
CONF_ACCESS_TOKEN: str
CONF_ADDRESS: str
CONF_AFTER: str
CONF_ALIAS: str
CONF_ALLOWLIST_EXTERNAL_URLS: str
CONF_API_KEY: str
CONF_API_TOKEN: str
CONF_API_VERSION: str
CONF_ARMING_TIME: str
CONF_AT: str
CONF_ATTRIBUTE: str
CONF_AUTH_MFA_MODULES: str
CONF_AUTH_PROVIDERS: str
CONF_AUTHENTICATION: str
CONF_BASE: str
CONF_BEFORE: str
CONF_BELOW: str
CONF_BINARY_SENSORS: str
CONF_BRIGHTNESS: str
CONF_BROADCAST_ADDRESS: str
CONF_BROADCAST_PORT: str
CONF_CHOOSE: str
CONF_CLIENT_ID: str
CONF_CLIENT_SECRET: str
CONF_CODE: str
CONF_COLOR_TEMP: str
CONF_COMMAND: str
CONF_COMMAND_CLOSE: str
CONF_COMMAND_OFF: str
CONF_COMMAND_ON: str
CONF_COMMAND_OPEN: str
CONF_COMMAND_STATE: str
CONF_COMMAND_STOP: str
CONF_CONDITION: str
CONF_CONDITIONS: str
CONF_CONTINUE_ON_TIMEOUT: str
CONF_COUNT: str
CONF_COVERS: str
CONF_CURRENCY: str
CONF_CUSTOMIZE: str
CONF_CUSTOMIZE_DOMAIN: str
CONF_CUSTOMIZE_GLOB: str
CONF_DEFAULT: str
CONF_DELAY: str
CONF_DELAY_TIME: str
CONF_DESCRIPTION: str
CONF_DEVICE: str
CONF_DEVICES: str
CONF_DEVICE_CLASS: str
CONF_DEVICE_ID: str
CONF_DISARM_AFTER_TRIGGER: str
CONF_DISCOVERY: str
CONF_DISKS: str
CONF_DISPLAY_CURRENCY: str
CONF_DISPLAY_OPTIONS: str
CONF_DOMAIN: str
CONF_DOMAINS: str
CONF_EFFECT: str
CONF_ELEVATION: str
CONF_EMAIL: str
CONF_ENTITIES: str
CONF_ENTITY_ID: str
CONF_ENTITY_NAMESPACE: str
CONF_ENTITY_PICTURE_TEMPLATE: str
CONF_EVENT: str
CONF_EVENT_DATA: str
CONF_EVENT_DATA_TEMPLATE: str
CONF_EXCLUDE: str
CONF_EXTERNAL_URL: str
CONF_FILENAME: str
CONF_FILE_PATH: str
CONF_FOR: str
CONF_FORCE_UPDATE: str
CONF_FRIENDLY_NAME: str
CONF_FRIENDLY_NAME_TEMPLATE: str
CONF_HEADERS: str
CONF_HOST: str
CONF_HOSTS: str
CONF_HS: str
CONF_ICON: str
CONF_ICON_TEMPLATE: str
CONF_ID: str
CONF_INCLUDE: str
CONF_INTERNAL_URL: str
CONF_IP_ADDRESS: str
CONF_LATITUDE: str
CONF_LEGACY_TEMPLATES: str
CONF_LIGHTS: str
CONF_LONGITUDE: str
CONF_MAC: str
CONF_MAXIMUM: str
CONF_MEDIA_DIRS: str
CONF_METHOD: str
CONF_MINIMUM: str
CONF_MODE: str
CONF_MONITORED_CONDITIONS: str
CONF_MONITORED_VARIABLES: str
CONF_NAME: str
CONF_OFFSET: str
CONF_OPTIMISTIC: str
CONF_PACKAGES: str
CONF_PARAMS: str
CONF_PASSWORD: str
CONF_PATH: str
CONF_PAYLOAD: str
CONF_PAYLOAD_OFF: str
CONF_PAYLOAD_ON: str
CONF_PENDING_TIME: str
CONF_PIN: str
CONF_PLATFORM: str
CONF_PORT: str
CONF_PREFIX: str
CONF_PROFILE_NAME: str
CONF_PROTOCOL: str
CONF_PROXY_SSL: str
CONF_QUOTE: str
CONF_RADIUS: str
CONF_RECIPIENT: str
CONF_REGION: str
CONF_REPEAT: str
CONF_RESOURCE: str
CONF_RESOURCES: str
CONF_RESOURCE_TEMPLATE: str
CONF_RGB: str
CONF_ROOM: str
CONF_SCAN_INTERVAL: str
CONF_SCENE: str
CONF_SELECTOR: str
CONF_SENDER: str
CONF_SENSORS: str
CONF_SENSOR_TYPE: str
CONF_SEQUENCE: str
CONF_SERVICE: str
CONF_SERVICE_DATA: str
CONF_SERVICE_TEMPLATE: str
CONF_SHOW_ON_MAP: str
CONF_SLAVE: str
CONF_SOURCE: str
CONF_SSL: str
CONF_STATE: str
CONF_STATE_TEMPLATE: str
CONF_STRUCTURE: str
CONF_SWITCHES: str
CONF_TARGET: str
CONF_TEMPERATURE_UNIT: str
CONF_TIMEOUT: str
CONF_TIME_ZONE: str
CONF_TOKEN: str
CONF_TRIGGER_TIME: str
CONF_TTL: str
CONF_TYPE: str
CONF_UNIQUE_ID: str
CONF_UNIT_OF_MEASUREMENT: str
CONF_UNIT_SYSTEM: str
CONF_UNTIL: str
CONF_URL: str
CONF_USERNAME: str
CONF_VALUE_TEMPLATE: str
CONF_VARIABLES: str
CONF_VERIFY_SSL: str
CONF_WAIT_FOR_TRIGGER: str
CONF_WAIT_TEMPLATE: str
CONF_WEBHOOK_ID: str
CONF_WEEKDAY: str
CONF_WHILE: str
CONF_WHITELIST: str
CONF_ALLOWLIST_EXTERNAL_DIRS: str
LEGACY_CONF_WHITELIST_EXTERNAL_DIRS: str
CONF_WHITE_VALUE: str
CONF_XY: str
CONF_ZONE: str
EVENT_CALL_SERVICE: str
EVENT_COMPONENT_LOADED: str
EVENT_CORE_CONFIG_UPDATE: str
EVENT_HOMEASSISTANT_CLOSE: str
EVENT_HOMEASSISTANT_START: str
EVENT_HOMEASSISTANT_STARTED: str
EVENT_HOMEASSISTANT_STOP: str
EVENT_HOMEASSISTANT_FINAL_WRITE: str
EVENT_LOGBOOK_ENTRY: str
EVENT_SERVICE_REGISTERED: str
EVENT_SERVICE_REMOVED: str
EVENT_STATE_CHANGED: str
EVENT_THEMES_UPDATED: str
EVENT_TIMER_OUT_OF_SYNC: str
EVENT_TIME_CHANGED: str
DEVICE_CLASS_BATTERY: str
DEVICE_CLASS_CO: str
DEVICE_CLASS_CO2: str
DEVICE_CLASS_HUMIDITY: str
DEVICE_CLASS_ILLUMINANCE: str
DEVICE_CLASS_SIGNAL_STRENGTH: str
DEVICE_CLASS_TEMPERATURE: str
DEVICE_CLASS_TIMESTAMP: str
DEVICE_CLASS_PRESSURE: str
DEVICE_CLASS_POWER: str
DEVICE_CLASS_CURRENT: str
DEVICE_CLASS_ENERGY: str
DEVICE_CLASS_POWER_FACTOR: str
DEVICE_CLASS_VOLTAGE: str
STATE_ON: str
STATE_OFF: str
STATE_HOME: str
STATE_NOT_HOME: str
STATE_UNKNOWN: str
STATE_OPEN: str
STATE_OPENING: str
STATE_CLOSED: str
STATE_CLOSING: str
STATE_PLAYING: str
STATE_PAUSED: str
STATE_IDLE: str
STATE_STANDBY: str
STATE_ALARM_DISARMED: str
STATE_ALARM_ARMED_HOME: str
STATE_ALARM_ARMED_AWAY: str
STATE_ALARM_ARMED_NIGHT: str
STATE_ALARM_ARMED_CUSTOM_BYPASS: str
STATE_ALARM_PENDING: str
STATE_ALARM_ARMING: str
STATE_ALARM_DISARMING: str
STATE_ALARM_TRIGGERED: str
STATE_LOCKED: str
STATE_UNLOCKED: str
STATE_UNAVAILABLE: str
STATE_OK: str
STATE_PROBLEM: str
ATTR_ATTRIBUTION: str
ATTR_CREDENTIALS: str
ATTR_NOW: str
ATTR_DATE: str
ATTR_TIME: str
ATTR_SECONDS: str
ATTR_DOMAIN: str
ATTR_SERVICE: str
ATTR_SERVICE_DATA: str
ATTR_ID: str
ATTR_NAME: str
ATTR_ENTITY_ID: str
ATTR_AREA_ID: str
ATTR_DEVICE_ID: str
ATTR_FRIENDLY_NAME: str
ATTR_ENTITY_PICTURE: str
ATTR_ICON: str
ATTR_UNIT_OF_MEASUREMENT: str
CONF_UNIT_SYSTEM_METRIC: str
CONF_UNIT_SYSTEM_IMPERIAL: str
ATTR_VOLTAGE: str
ATTR_LOCATION: str
ATTR_MODE: str
ATTR_BATTERY_CHARGING: str
ATTR_BATTERY_LEVEL: str
ATTR_WAKEUP: str
ATTR_CODE: str
ATTR_CODE_FORMAT: str
ATTR_COMMAND: str
ATTR_ARMED: str
ATTR_LOCKED: str
ATTR_TRIPPED: str
ATTR_LAST_TRIP_TIME: str
ATTR_HIDDEN: str
ATTR_LATITUDE: str
ATTR_LONGITUDE: str
ATTR_GPS_ACCURACY: str
ATTR_ASSUMED_STATE: str
ATTR_STATE: str
ATTR_EDITABLE: str
ATTR_OPTION: str
ATTR_RESTORED: str
ATTR_SUPPORTED_FEATURES: str
ATTR_DEVICE_CLASS: str
ATTR_TEMPERATURE: str
POWER_WATT: str
POWER_KILO_WATT: str
VOLT: str
ENERGY_WATT_HOUR: str
ENERGY_KILO_WATT_HOUR: str
ELECTRICAL_CURRENT_AMPERE: str
ELECTRICAL_VOLT_AMPERE: str
DEGREE: str
CURRENCY_EURO: str
CURRENCY_DOLLAR: str
CURRENCY_CENT: str
TEMP_CELSIUS: str
TEMP_FAHRENHEIT: str
TEMP_KELVIN: str
TIME_MICROSECONDS: str
TIME_MILLISECONDS: str
TIME_SECONDS: str
TIME_MINUTES: str
TIME_HOURS: str
TIME_DAYS: str
TIME_WEEKS: str
TIME_MONTHS: str
TIME_YEARS: str
LENGTH_MILLIMETERS: str
LENGTH_CENTIMETERS: str
LENGTH_METERS: str
LENGTH_KILOMETERS: str
LENGTH_INCHES: str
LENGTH_FEET: str
LENGTH_YARD: str
LENGTH_MILES: str
FREQUENCY_HERTZ: str
FREQUENCY_GIGAHERTZ: str
PRESSURE_PA: str
PRESSURE_HPA: str
PRESSURE_BAR: str
PRESSURE_MBAR: str
PRESSURE_INHG: str
PRESSURE_PSI: str
VOLUME_LITERS: str
VOLUME_MILLILITERS: str
VOLUME_CUBIC_METERS: str
VOLUME_CUBIC_FEET: str
VOLUME_GALLONS: str
VOLUME_FLUID_OUNCE: str
VOLUME_FLOW_RATE_CUBIC_METERS_PER_HOUR: str
VOLUME_FLOW_RATE_CUBIC_FEET_PER_MINUTE: str
AREA_SQUARE_METERS: str
MASS_GRAMS: str
MASS_KILOGRAMS: str
MASS_MILLIGRAMS: str
MASS_MICROGRAMS: str
MASS_OUNCES: str
MASS_POUNDS: str
CONDUCTIVITY: str
LIGHT_LUX: str
UV_INDEX: str
PERCENTAGE: str
IRRADIATION_WATTS_PER_SQUARE_METER: str
PRECIPITATION_MILLIMETERS_PER_HOUR: str
CONCENTRATION_MICROGRAMS_PER_CUBIC_METER: str
CONCENTRATION_MILLIGRAMS_PER_CUBIC_METER: str
CONCENTRATION_PARTS_PER_CUBIC_METER: str
CONCENTRATION_PARTS_PER_MILLION: str
CONCENTRATION_PARTS_PER_BILLION: str
SPEED_MILLIMETERS_PER_DAY: str
SPEED_INCHES_PER_DAY: str
SPEED_METERS_PER_SECOND: str
SPEED_INCHES_PER_HOUR: str
SPEED_KILOMETERS_PER_HOUR: str
SPEED_MILES_PER_HOUR: str
SIGNAL_STRENGTH_DECIBELS: str
SIGNAL_STRENGTH_DECIBELS_MILLIWATT: str
DATA_BITS: str
DATA_KILOBITS: str
DATA_MEGABITS: str
DATA_GIGABITS: str
DATA_BYTES: str
DATA_KILOBYTES: str
DATA_MEGABYTES: str
DATA_GIGABYTES: str
DATA_TERABYTES: str
DATA_PETABYTES: str
DATA_EXABYTES: str
DATA_ZETTABYTES: str
DATA_YOTTABYTES: str
DATA_KIBIBYTES: str
DATA_MEBIBYTES: str
DATA_GIBIBYTES: str
DATA_TEBIBYTES: str
DATA_PEBIBYTES: str
DATA_EXBIBYTES: str
DATA_ZEBIBYTES: str
DATA_YOBIBYTES: str
DATA_RATE_BITS_PER_SECOND: str
DATA_RATE_KILOBITS_PER_SECOND: str
DATA_RATE_MEGABITS_PER_SECOND: str
DATA_RATE_GIGABITS_PER_SECOND: str
DATA_RATE_BYTES_PER_SECOND: str
DATA_RATE_KILOBYTES_PER_SECOND: str
DATA_RATE_MEGABYTES_PER_SECOND: str
DATA_RATE_GIGABYTES_PER_SECOND: str
DATA_RATE_KIBIBYTES_PER_SECOND: str
DATA_RATE_MEBIBYTES_PER_SECOND: str
DATA_RATE_GIBIBYTES_PER_SECOND: str
SERVICE_HOMEASSISTANT_STOP: str
SERVICE_HOMEASSISTANT_RESTART: str
SERVICE_TURN_ON: str
SERVICE_TURN_OFF: str
SERVICE_TOGGLE: str
SERVICE_RELOAD: str
SERVICE_VOLUME_UP: str
SERVICE_VOLUME_DOWN: str
SERVICE_VOLUME_MUTE: str
SERVICE_VOLUME_SET: str
SERVICE_MEDIA_PLAY_PAUSE: str
SERVICE_MEDIA_PLAY: str
SERVICE_MEDIA_PAUSE: str
SERVICE_MEDIA_STOP: str
SERVICE_MEDIA_NEXT_TRACK: str
SERVICE_MEDIA_PREVIOUS_TRACK: str
SERVICE_MEDIA_SEEK: str
SERVICE_REPEAT_SET: str
SERVICE_SHUFFLE_SET: str
SERVICE_ALARM_DISARM: str
SERVICE_ALARM_ARM_HOME: str
SERVICE_ALARM_ARM_AWAY: str
SERVICE_ALARM_ARM_NIGHT: str
SERVICE_ALARM_ARM_CUSTOM_BYPASS: str
SERVICE_ALARM_TRIGGER: str
SERVICE_LOCK: str
SERVICE_UNLOCK: str
SERVICE_OPEN: str
SERVICE_CLOSE: str
SERVICE_CLOSE_COVER: str
SERVICE_CLOSE_COVER_TILT: str
SERVICE_OPEN_COVER: str
SERVICE_OPEN_COVER_TILT: str
SERVICE_SET_COVER_POSITION: str
SERVICE_SET_COVER_TILT_POSITION: str
SERVICE_STOP_COVER: str
SERVICE_STOP_COVER_TILT: str
SERVICE_TOGGLE_COVER_TILT: str
SERVICE_SELECT_OPTION: str
SERVER_PORT: int
URL_ROOT: str
URL_API: str
URL_API_STREAM: str
URL_API_CONFIG: str
URL_API_DISCOVERY_INFO: str
URL_API_STATES: str
URL_API_STATES_ENTITY: str
URL_API_EVENTS: str
URL_API_EVENTS_EVENT: str
URL_API_SERVICES: str
URL_API_SERVICES_SERVICE: str
URL_API_COMPONENTS: str
URL_API_ERROR_LOG: str
URL_API_LOG_OUT: str
URL_API_TEMPLATE: str
HTTP_OK: int
HTTP_CREATED: int
HTTP_ACCEPTED: int
HTTP_MOVED_PERMANENTLY: int
HTTP_BAD_REQUEST: int
HTTP_UNAUTHORIZED: int
HTTP_FORBIDDEN: int
HTTP_NOT_FOUND: int
HTTP_METHOD_NOT_ALLOWED: int
HTTP_UNPROCESSABLE_ENTITY: int
HTTP_TOO_MANY_REQUESTS: int
HTTP_INTERNAL_SERVER_ERROR: int
HTTP_BAD_GATEWAY: int
HTTP_SERVICE_UNAVAILABLE: int
HTTP_BASIC_AUTHENTICATION: str
HTTP_DIGEST_AUTHENTICATION: str
HTTP_HEADER_X_REQUESTED_WITH: str
CONTENT_TYPE_JSON: str
CONTENT_TYPE_MULTIPART: str
CONTENT_TYPE_TEXT_PLAIN: str
RESTART_EXIT_CODE: int
UNIT_NOT_RECOGNIZED_TEMPLATE: str
LENGTH: str
MASS: str
PRESSURE: str
VOLUME: str
TEMPERATURE: str
SPEED_MS: str
ILLUMINANCE: str
WEEKDAYS: Any
PRECISION_WHOLE: int
PRECISION_HALVES: float
PRECISION_TENTHS: float
CLOUD_NEVER_EXPOSED_ENTITIES: Any
CAST_APP_ID_HOMEASSISTANT: str



from homeassistant.const import DEVICE_CLASS_BATTERY as DEVICE_CLASS_BATTERY, DEVICE_CLASS_CO as DEVICE_CLASS_CO, DEVICE_CLASS_CO2 as DEVICE_CLASS_CO2, DEVICE_CLASS_CURRENT as DEVICE_CLASS_CURRENT, DEVICE_CLASS_ENERGY as DEVICE_CLASS_ENERGY, DEVICE_CLASS_HUMIDITY as DEVICE_CLASS_HUMIDITY, DEVICE_CLASS_ILLUMINANCE as DEVICE_CLASS_ILLUMINANCE, DEVICE_CLASS_POWER as DEVICE_CLASS_POWER, DEVICE_CLASS_POWER_FACTOR as DEVICE_CLASS_POWER_FACTOR, DEVICE_CLASS_PRESSURE as DEVICE_CLASS_PRESSURE, DEVICE_CLASS_SIGNAL_STRENGTH as DEVICE_CLASS_SIGNAL_STRENGTH, DEVICE_CLASS_TEMPERATURE as DEVICE_CLASS_TEMPERATURE, DEVICE_CLASS_TIMESTAMP as DEVICE_CLASS_TIMESTAMP, DEVICE_CLASS_VOLTAGE as DEVICE_CLASS_VOLTAGE
from homeassistant.helpers.config_validation import PLATFORM_SCHEMA as PLATFORM_SCHEMA, PLATFORM_SCHEMA_BASE as PLATFORM_SCHEMA_BASE
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.entity_component import EntityComponent as EntityComponent
from typing import Any

_LOGGER: Any
DOMAIN: str
ENTITY_ID_FORMAT: Any
SCAN_INTERVAL: Any
DEVICE_CLASSES: Any
DEVICE_CLASSES_SCHEMA: Any

async def async_setup(hass: Any, config: Any): ...
async def async_setup_entry(hass: Any, entry: Any): ...
async def async_unload_entry(hass: Any, entry: Any): ...

class SensorEntity(Entity): ...



from . import DOMAIN as DOMAIN
from homeassistant.components.automation import AutomationActionType as AutomationActionType
from homeassistant.components.device_automation import TRIGGER_BASE_SCHEMA as TRIGGER_BASE_SCHEMA
from homeassistant.const import CONF_DEVICE_ID as CONF_DEVICE_ID, CONF_DOMAIN as CONF_DOMAIN, CONF_ENTITY_ID as CONF_ENTITY_ID, CONF_FOR as CONF_FOR, CONF_PLATFORM as CONF_PLATFORM, CONF_TYPE as CONF_TYPE, STATE_IDLE as STATE_IDLE, STATE_OFF as STATE_OFF, STATE_ON as STATE_ON, STATE_PAUSED as STATE_PAUSED, STATE_PLAYING as STATE_PLAYING
from homeassistant.core import CALLBACK_TYPE as CALLBACK_TYPE, HomeAssistant as HomeAssistant
from homeassistant.helpers import entity_registry as entity_registry
from homeassistant.helpers.typing import ConfigType as ConfigType
from typing import Any

TRIGGER_TYPES: Any
TRIGGER_SCHEMA: Any

async def async_get_triggers(hass: HomeAssistant, device_id: str) -> list[dict]: ...
async def async_get_trigger_capabilities(hass: HomeAssistant, config: dict) -> dict: ...
async def async_attach_trigger(hass: HomeAssistant, config: ConfigType, action: AutomationActionType, automation_info: dict) -> CALLBACK_TYPE: ...



from .const import DOMAIN as DOMAIN
from .knx_entity import KnxEntity as KnxEntity
from datetime import datetime
from homeassistant.components.cover import ATTR_POSITION as ATTR_POSITION, ATTR_TILT_POSITION as ATTR_TILT_POSITION, CoverEntity as CoverEntity, DEVICE_CLASSES as DEVICE_CLASSES, DEVICE_CLASS_BLIND as DEVICE_CLASS_BLIND, SUPPORT_CLOSE as SUPPORT_CLOSE, SUPPORT_CLOSE_TILT as SUPPORT_CLOSE_TILT, SUPPORT_OPEN as SUPPORT_OPEN, SUPPORT_OPEN_TILT as SUPPORT_OPEN_TILT, SUPPORT_SET_POSITION as SUPPORT_SET_POSITION, SUPPORT_SET_TILT_POSITION as SUPPORT_SET_TILT_POSITION, SUPPORT_STOP as SUPPORT_STOP, SUPPORT_STOP_TILT as SUPPORT_STOP_TILT
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.event import async_track_utc_time_change as async_track_utc_time_change
from homeassistant.helpers.typing import ConfigType as ConfigType, DiscoveryInfoType as DiscoveryInfoType
from typing import Any, Callable, Iterable
from xknx.devices import Cover as XknxCover, Device as XknxDevice

async def async_setup_platform(hass: HomeAssistant, config: ConfigType, async_add_entities: Callable[[Iterable[Entity]], None], discovery_info: Union[DiscoveryInfoType, None]=...) -> None: ...

class KNXCover(KnxEntity, CoverEntity):
    _device: Any
    _unsubscribe_auto_updater: Any = ...
    def __init__(self, device: XknxCover) -> None: ...
    async def after_update_callback(self, device: XknxDevice) -> None: ...
    @property
    def device_class(self) -> Union[str, None]: ...
    @property
    def supported_features(self) -> int: ...
    @property
    def current_cover_position(self) -> Union[int, None]: ...
    @property
    def is_closed(self) -> Union[bool, None]: ...
    @property
    def is_opening(self) -> bool: ...
    @property
    def is_closing(self) -> bool: ...
    async def async_close_cover(self, **kwargs: Any) -> None: ...
    async def async_open_cover(self, **kwargs: Any) -> None: ...
    async def async_set_cover_position(self, **kwargs: Any) -> None: ...
    async def async_stop_cover(self, **kwargs: Any) -> None: ...
    @property
    def current_cover_tilt_position(self) -> Union[int, None]: ...
    async def async_set_cover_tilt_position(self, **kwargs: Any) -> None: ...
    async def async_open_cover_tilt(self, **kwargs: Any) -> None: ...
    async def async_close_cover_tilt(self, **kwargs: Any) -> None: ...
    async def async_stop_cover_tilt(self, **kwargs: Any) -> None: ...
    def start_auto_updater(self) -> None: ...
    def stop_auto_updater(self) -> None: ...
    def auto_updater_hook(self, now: datetime) -> None: ...



from . import GroupEntity as GroupEntity
from homeassistant.components.cover import ATTR_CURRENT_POSITION as ATTR_CURRENT_POSITION, ATTR_CURRENT_TILT_POSITION as ATTR_CURRENT_TILT_POSITION, ATTR_POSITION as ATTR_POSITION, ATTR_TILT_POSITION as ATTR_TILT_POSITION, CoverEntity as CoverEntity, DOMAIN as DOMAIN, PLATFORM_SCHEMA as PLATFORM_SCHEMA, SERVICE_CLOSE_COVER as SERVICE_CLOSE_COVER, SERVICE_CLOSE_COVER_TILT as SERVICE_CLOSE_COVER_TILT, SERVICE_OPEN_COVER as SERVICE_OPEN_COVER, SERVICE_OPEN_COVER_TILT as SERVICE_OPEN_COVER_TILT, SERVICE_SET_COVER_POSITION as SERVICE_SET_COVER_POSITION, SERVICE_SET_COVER_TILT_POSITION as SERVICE_SET_COVER_TILT_POSITION, SERVICE_STOP_COVER as SERVICE_STOP_COVER, SERVICE_STOP_COVER_TILT as SERVICE_STOP_COVER_TILT, SUPPORT_CLOSE as SUPPORT_CLOSE, SUPPORT_CLOSE_TILT as SUPPORT_CLOSE_TILT, SUPPORT_OPEN as SUPPORT_OPEN, SUPPORT_OPEN_TILT as SUPPORT_OPEN_TILT, SUPPORT_SET_POSITION as SUPPORT_SET_POSITION, SUPPORT_SET_TILT_POSITION as SUPPORT_SET_TILT_POSITION, SUPPORT_STOP as SUPPORT_STOP, SUPPORT_STOP_TILT as SUPPORT_STOP_TILT
from homeassistant.const import ATTR_ASSUMED_STATE as ATTR_ASSUMED_STATE, ATTR_ENTITY_ID as ATTR_ENTITY_ID, ATTR_SUPPORTED_FEATURES as ATTR_SUPPORTED_FEATURES, CONF_ENTITIES as CONF_ENTITIES, CONF_NAME as CONF_NAME, STATE_CLOSING as STATE_CLOSING, STATE_OPEN as STATE_OPEN, STATE_OPENING as STATE_OPENING
from homeassistant.core import CoreState as CoreState, State as State
from homeassistant.helpers.event import async_track_state_change_event as async_track_state_change_event
from typing import Any, Optional

KEY_OPEN_CLOSE: str
KEY_STOP: str
KEY_POSITION: str
DEFAULT_NAME: str

async def async_setup_platform(hass: Any, config: Any, async_add_entities: Any, discovery_info: Optional[Any] = ...) -> None: ...

class CoverGroup(GroupEntity, CoverEntity):
    _name: Any = ...
    _is_closed: bool = ...
    _is_closing: bool = ...
    _is_opening: bool = ...
    _cover_position: int = ...
    _tilt_position: Any = ...
    _supported_features: int = ...
    _assumed_state: bool = ...
    _entities: Any = ...
    _covers: Any = ...
    _tilts: Any = ...
    def __init__(self, name: Any, entities: Any) -> None: ...
    async def _update_supported_features_event(self, event: Any) -> None: ...
    async def async_update_supported_features(self, entity_id: str, new_state: Union[State, None], update_state: bool=...) -> None: ...
    async def async_added_to_hass(self) -> None: ...
    @property
    def name(self): ...
    @property
    def assumed_state(self): ...
    @property
    def supported_features(self): ...
    @property
    def is_closed(self): ...
    @property
    def is_opening(self): ...
    @property
    def is_closing(self): ...
    @property
    def current_cover_position(self) -> Union[int, None]: ...
    @property
    def current_cover_tilt_position(self): ...
    @property
    def extra_state_attributes(self): ...
    async def async_open_cover(self, **kwargs: Any) -> None: ...
    async def async_close_cover(self, **kwargs: Any) -> None: ...
    async def async_stop_cover(self, **kwargs: Any) -> None: ...
    async def async_set_cover_position(self, **kwargs: Any) -> None: ...
    async def async_open_cover_tilt(self, **kwargs: Any) -> None: ...
    async def async_close_cover_tilt(self, **kwargs: Any) -> None: ...
    async def async_stop_cover_tilt(self, **kwargs: Any) -> None: ...
    async def async_set_cover_tilt_position(self, **kwargs: Any) -> None: ...
    async def async_update(self) -> None: ...
